/*
* -cr_beg-
* ********************************************************************
* IBM Confidential
* OCO Source Materials
* 5724-U38, 5724-U39, 5724-U48, 5655-V17, 5655-V19, 5655-V35, 5655-V37
* (C) Copyright IBM Corp. 1996,2009,2010
* The source code for this program is not published or otherwise
* divested of its trade secrets, irrespective of what has
* been deposited with the U.S. Copyright Office.
* ********************************************************************
* -cr_end-
*/

#ifndef DXXC_H
#define DXXC_H
/*--------------------------------------------------------------------*
 *                                                                    *
 * HEADER FILE NAME = DXXC                                            *
 *                                                                    *
 * CLASSIFICATION = PUBLIC file part of Published Component Interface *
 *                                                                    *
 * HEAD FILE TYPE = Component Public Interface declaration            *
 *                                                                    *
 * PRODUCT = Non-External Components                                  *
 *                                                                    *
 * COMPONENT = Data Extract                                           *
 *                                                                    *
 * DESCRIPTIVE NAME = Data Extract Public interface elements          *
 *                                                                    *
 *                                                                    *
 * VERSION = 1.0                                                      *
 *                                                                    *
 * Function: This header file is for use by components outside the    *
 *           DXX subsystem.  It includes all the definitions needed   *
 *           to interface with the DXX Subsystem.  Included are:      *
 *                                                                    *
 *               DXX External Structures                              *
 *               DXX External Constants                               *
 *               The DXX Function Vector                              *
 *               DXX Function call macros                             *
 *                                                                    *
 *--------------------------------------------------------------------*
 *                                                                    *
 * NOTES :                                                            *
 *    DEPENDENCIES = NEC Framework Specification                      *
 *    RESTRICTIONS = None                                             *
 *    REGISTER CONVENTIONS = Standard for the platform                *
 *    PATCH LABEL = N/A                                               *
 *    PROCESSOR = Microsoft Visual C++, IBM Visual Age C++, SAS/C     *
 *    ATTRIBUTES = Read only, header file                             *
 *                                                                    *
 *    AUTHOR = D. Hoeschele                                           *
 *                                                                    *
 * CHANGE ACTIVITY :                                                  *
 *   Vers  ID     Date    Change Description                          *
 *    1.0  DH   02/03/96  Initial creation                            *
 *    2.0  NB   12/10/99  Added support for additional DDL objects    *
 *    2.1  EL   10/11/02  Added support for additional DDL objects    *
 *                                                                    *
 *--------------------------------------------------------------------*
 * RTC/Jazz change log
 *
 * 15 Jul 2011 osheat 8.1.0 4034 WI 28046
 * Report Teradata account ID and client character set.
 *
 * 29 June 2011 TulasiU 8.1.0 4029 RTC 18001
 * Changes to support Metering Statistics
 *
 * 26 May 2011 SagarB 8.1.0 4020 WI 21884
 * Teradata Native support
 *
 * 10 April 2011 SagarB 8.1.0 RTC 22144.
 * Uplift the Maximum Row limit supported in Optim
 * Phase 1 - Limited to Archive/Extract,insert,Delete,convert and Load.
 *
 * 17 Jun 2010 SagarB 7.3.0 WI #3682
 * Changes to incorporate Compression Enhancement
 *
 * 09 April 2010 SagarB  7.3.0 3522 RTC WI 73280
 * Added Index return error code DXERR_IDX_INTERNAL_ERROR.
 *
 * 28 Dec 2009 RTandon1 7.2.1 3108 WI 4958: Number Ignore Option does not
 * work with Oracle 10.2.
 *
 * 31 Aug 2009 rtandon1 7.2.0 3012
 * Changes for Long Computer Name
 *
 *
 * Modified:
 *
 *    $Author:   OPTIMBLD  $
 *    $Date:   22 Oct 2008 16:31:12  $
 *    $Revision:   1.2  $
 *    $Log:   Y:\RTWIN\66\ARCHIVES\DXXC.H_v  $
 *
 *    Rev 1.2   22 Oct 2008 16:31:12   OPTIMBLD
 * Remove keyword flagged by Source Code Scanning Utility "S O F T E C H"
 *
 *    Rev 1.1   21 Oct 2008 09:47:46   OPTIMBLD
 * Remove keyword flagged by Source Code Scanning Utility "B A S E D  O N"
 *
 *    Rev 1.0   07 Apr 2008 08:15:54   BOBO
 * Initial revision.
 *
 *--------------------------------------------------------------------*/

#ifdef PST_CPLUS          /* Don't mangle names */
extern "C" {
#endif


/* include definition of new OSIO parm data */
#include <DALCOSIX.H>



/* Extract Function Codes */
#define DX_FUNC_SHOW_STEPS         'S'
#define DX_FUNC_VALIDATE           'V'
#define DX_FUNC_VALIDATE_INDEXES   'I'
#define DX_FUNC_COMPILE            'C'
#define DX_FUNC_COMPILE_AND_EXEC   'A'
#define DX_FUNC_EXEC               'X'


/*
The Function Codes are:
    DX_FUNC_SHOW_STEPS        - Analyze the extract and return DX
                                Explain Structures as it does now
                                for MVS.

    DX_FUNC_VALIDATE_INDEXES  - Analyze the extract and return
                                info about Relationship traversals
                                and available indexes.

    DX_FUNC_VALIDATE          - Fully validate an extract, with optional
                                SQL validation.  No Extract Object
                                structures are returned.

    DX_FUNC_COMPILE           - Fully validate an extract, with optional
                                SQL validation.  Extract Object
                                structures are returned in the form of
                                one relocatable chunk of data.  It may
                                not go through a code page translation.
                                Object data is returned only if there
                                are no errors or there are only DBMS
                                authorization errors.

    DX_FUNC_COMPILE_AND_EXEC  - Validate, Compile, and execute an
                                extract.  No object data is returned.

    DX_FUNC_EXEC              - Execute a previously compiled extract.
*/



#define DX_OVERRIDE_NONE      0
#define DX_OVERRIDE_PRESENT   1

#define DX_DATA_ONLY          0
#define DX_DDL_ONLY           1
#define DX_DATA_AND_DDL       2

#define DX_ACTION_ERR_STOP        1
#define DX_ACTION_ERR_SKIP_ROW    2
#define DX_ACTION_ERR_EXTRACT_ROW 3

#define DX_DDL_COPIED          1
#define DX_NO_DDL_TO_COPY      2
#define DX_MVS_DDL_NOT_COPIED  3


/* Codes for the Relationship Num to Char Conversion Option */
#define  DX_TRANS_NC_REL_STD   0  /* Right justify num, zero fill  */
#define  DX_TRANS_NC_REL_ZAP1  1  /* Right justify num, space fill */
#define  DX_TRANS_NC_REL_ZAP2  2  /* Left justify num, space fill  */

/* Constants to direct how extract accesses a table */
#define DX_SCAN_NEVER        1
#define DX_SCAN_RARELY       2
#define DX_SCAN_SOMETIMES    3
#define DX_FORCE_NONE        0
#define DX_FORCE_KEYLOOKUP   1
#define DX_FORCE_SCAN        2

/* KSB: 25/05/2011 - Teradata Native support.
   Teradata will have dual connection mode for table access.
   TPT or CLI.*/
#define DX_TERA_CONN_MODE_CLI   1
#define DX_TERA_CONN_MODE_TPT   2

/* Constants for Archive Action Extract Status.  One of these
   values will be in the ProcessStatus Host variable */
#define  DX_STATUS_SUCCESS            0
#define  DX_STATUS_CANCEL             1
#define  DX_STATUS_FATAL_DBMS_ERROR   2
#define  DX_STATUS_ACTION_ABORT       3
#define  DX_STATUS_FATAL_ERROR        4


/* Extract Return Codes, range 15361 - 16384 */
#define DX_ERROR     15500         /* Some error was detected, check the
                                      DXCA Structure for error message
                                      numbers and aux info */

/* Some error message numbers that are put into the DXCA */
#define DXERR_NO_TABLES           15501  /* No tbls in the Input List */
#define DXERR_INVALID_START_TABLE 15502
#define DXERR_REL_CHILD_TABLE_NOT_IN_LIST 15503
                                           /* Tok1 = DBAL.CID.Tbl.Rel */
#define DXERR_REL_PAR_TABLE_NOT_IN_LIST 15504
                                           /* Tok1 = DBAL.CID.Tbl.Rel */
#define DXERR_DBALIAS_UNKNOWN    15505     /* Tok1 = DBAlias          */
#define DXERR_TBL_NOT_FOUND      15506     /* Tok1 = DBAL.CID.Tbl     */
#define DX_CANCEL                15507     /* Product asked for abort */
#define DXERR_PK_COL_NOT_FOUND   15508     /* Tok1 = DBAL.CID.Tbl
                                              Tok2 = PKColName        */

#define DXERR_REL_NOT_FOUND      15509     /* Tok1 = DBAL.CID.Tbl.Rel */
#define DXERR_DUP_TABLES         15510     /* Tok1 = DBAL.CID1.Tbl1   */
                                           /* Tok2 = DBAL.CID2.Tbl2   */
#define DXERR_INVALID_OVERRIDE_TBL 15512   /* Tok1 = DBAL.CID.Tbl     */
#define DXERR_NOFM_COL_NOT_FOUND   15513   /* Tok1 = DBAL.CID.Tbl
                                              Tok2 = NofM ColName     */
#define DXERR_FILE_FULL            15514   /* Output Extract File Full*/
#define DXERR_MAX_ROW_LIMIT_EXCEEDED 15515 /* User row limit exceeded */
#define DXERR_INVALID_XF           15516   /* Invalid Xtract File     */
#define DXERR_OPENERR_XF           15517   /* XFB couldn't open File  */
#define DXERR_DUP_PK               15518   /* XFB detected a duplicate
                                              PK value when building
                                              its PK Index.           */
#define DXERR_DBERR                15519   /* A RDBERR_DBERR occured,
                                              info in the RDB Thrd CB.
                                              Tok1 may = DBAL.CID.Tbl */
#define DXERR_INVALID_INPUT_PK     15520   /* Tok1 = DBAL.CID.Tbl     */
#define DXERR_REMOTE_TABLE         15521   /* Tok1=DBAL.CID.Tbl       */
#define DXERR_INVALID_VIEW         15522   /* Tok1=DBAL.CID.ViewName  */
#define DXERR_REMOTE_TABLE_IN_REL  15523   /* Tok1=DBAL.CID.Tbl.Rel   */
#define DXERR_TBL_VALIDATION_ERR   15524   /* When this error is in
                                              the DXCA, there are one
                                              or more errors reported
                                              in the DX_INPUT_TABLE
                                              structs.                */
#define DXERR_INVALID_REL        15525     /* Tok1 = DBAL.CID.Tbl.Rel */
#define DXERR_VALIDATE_STMT_DBERR 15526    /* A RDBERR_DBERR occured
                                              while validating a Table's
                                              SQL Select Statement. Info
                                              is in the RDB Thrd CB.
                                              Tok1 may = DBAL.CID.Tbl */
#define DXERR_NOFM_COL_TOO_LARGE   15527   /* Tok1 = DBAL.CID.Tbl
                                              Tok2 = NofM ColName     */
#define DXERR_SORT_KEY_TOO_LARGE   15528   /* Sort Key > 256 bytes
                                              Tok1 = DBAL.CID.Tbl     */
#define DXERR_SORT_COL_NOT_FOUND   15529   /* Sort Col not found in
                                              Table Col List
                                              Tok1 = DBAL.CID.Tbl
                                              Tok2 = Sort ColName     */
#define DXERR_OBJ_VALIDATION_ERR   15530   /* When this error is in
                                              the DXCA, there are one
                                              or more errors reported
                                              in the DX_EXPLICIT_OBJECT_LIST
                                              structs.                */
#define DXERR_INDEX_COL_UNKNOWN    15531 /* Tok1 = DBAL.CID.Tbl
                                            Tok2 = IndexName
                                            Tok3 = ColName      */
#define DXERR_INDEX_COL_INVALID    15532 /* Tok1 = DBAL.CID.Tbl
                                            Tok2 = IndexName
                                            Tok3 = ColName      */
#define DXERR_INDEX_TOO_LONG       15533 /* Tok1 = DBAL.CID.Tbl
                                            Tok2 = IndexName    */

/* Before Extract Action Errors */
#define DXERR_BF_XTR_ACTION_SYNTAX        15534 /* Invalid Action Syntax
                                                   Tok1 = invalid token */
#define DXERR_BF_XTR_ACTION_NOT_SUPPORTED 15535 /* Action not supported */
#define DXERR_BF_XTR_ACTION_HV_NOT_FOUND  15536 /* Action Host Var not found
                                                   Tok1 = invalid Host Var */
#define DXERR_BF_XTR_ACTION_VAL_RDBERR    15537 /* Action DBMS validat'n err*/

/* After Extract Action Errors */
#define DXERR_AF_XTR_ACTION_SYNTAX        15538 /* Invalid Action Syntax
                                                   Tok1 = invalid token */
#define DXERR_AF_XTR_ACTION_NOT_SUPPORTED 15539 /* Action not supported */
#define DXERR_AF_XTR_ACTION_HV_NOT_FOUND  15540 /* Action Host Var not found
                                                   Tok1 = invalid Host Var */
#define DXERR_AF_XTR_ACTION_VAL_RDBERR    15541 /* Action DBMS validat'n err*/

/* Before Table Action Errors */
#define DXERR_BF_TBL_ACTION_SYNTAX        15542 /* Invalid Action Syntax
                                                   Tok1 = invalid token
                                                   Tok2 = DBAL.CID.Tbl  */
#define DXERR_BF_TBL_ACTION_NOT_SUPPORTED 15543 /* Action not supported
                                                   Tok1 = DBAL.CID.Tbl  */
#define DXERR_BF_TBL_ACTION_HV_NOT_FOUND  15544 /* Action Host Var not found
                                                   Tok1 = invalid Host Var
                                                   Tok2 = DBAL.CID.Tbl  */
#define DXERR_BF_TBL_ACTION_VAL_RDBERR    15545 /* Action DBMS validat'n err
                                                   Tok1 = DBAL.CID.Tbl  */

/* After Table Action Errors */
#define DXERR_AF_TBL_ACTION_SYNTAX        15546 /* Invalid Action Syntax
                                                   Tok1 = invalid token
                                                   Tok2 = DBAL.CID.Tbl  */
#define DXERR_AF_TBL_ACTION_NOT_SUPPORTED 15547 /* Action not supported
                                                   Tok1 = DBAL.CID.Tbl  */
#define DXERR_AF_TBL_ACTION_HV_NOT_FOUND  15548 /* Action Host Var not found
                                                   Tok1 = invalid Host Var
                                                   Tok2 = DBAL.CID.Tbl  */
#define DXERR_AF_TBL_ACTION_VAL_RDBERR    15549 /* Action DBMS validat'n err
                                                   Tok1 = DBAL.CID.Tbl  */
/* Before Row Action Errors */
#define DXERR_BF_ROW_ACTION_SYNTAX        15550 /* Invalid Action Syntax
                                                   Tok1 = invalid token
                                                   Tok2 = DBAL.CID.Tbl  */
#define DXERR_BF_ROW_ACTION_NOT_SUPPORTED 15551 /* Action not supported
                                                   Tok1 = DBAL.CID.Tbl  */
#define DXERR_BF_ROW_ACTION_HV_NOT_FOUND  15552 /* Action Host Var not found
                                                   Tok1 = invalid Host Var
                                                   Tok2 = DBAL.CID.Tbl  */
#define DXERR_BF_ROW_ACTION_VAL_RDBERR    15553 /* Action DBMS validat'n err
                                                   Tok1 = DBAL.CID.Tbl  */

#define DXERR_ACTION_RDBERR               15554 /* Action execution resulted
                                                   in an RDBERR.  Info in
                                                   the RDB Thrd CB.
                                                   Tok1 = "Before Extract",
                                                          "After Extract",
                                                          "Before Table",
                                                          "After Table",
                                                          or "Before Row",
                                                   Tok2 = DBAL.CID.Tbl if
                                                          a tbl or row err */
#define DXERR_ACTION_ABORT                15555 /* Action Aborted Extract
                                                   Tok1 = "Before Extract",
                                                          "After Extract",
                                                          "Before Table",
                                                          "After Table",
                                                          or "Before Row",
                                                   Tok2 = DBAL.CID.Tbl if
                                                          a tbl or row err */

#define DXERR_IXFILE_OPEN_ERR             15556 /* Error openning Index File */
#define DXERR_TMPFILE_OPEN_ERR            15557 /* Error openning Index Temp
                                                   work file            */
#define DXERR_IDX_TMPFILE_IOERROR       15558 /* IO error on Index Temp File */
#define DXERR_IDX_TMPFILE_FULL          15559 /* Index Temp File Full   */
#define DXERR_IDX_FILE_IOERROR          15560 /* IO error on Index File */
#define DXERR_IDX_FILE_FULL             15561 /* Index File Full        */
#define DXERR_IDX_SORT_FAILURE          15562 /* Index Sort failure     */
#define DXERR_INVALID_WHERE_CLAUSE      15563 /* Invalid Table WHERE clause */
#define DXERR_RBA_FILE_FULL             15564 /* XFB Temp File Full      */
#define DXERR_OSIO_ERR                  15565 /* A, OSIOERR_OSIO_ERR
                                                 occured, call the
                                                 DalGetLastOSIOError()
                                                 to get more info.      */
#define DXERR_INVALID_HINT              15566 /* Invalid Optimization
                                                 comment in the Where
                                                 clause                 */
#define DXERR_ATTACHMENT_NOT_FOUND      15567 /* File attachment not found
                                                 in any of the input directories
                                                 Tok1 = DBAL.CID.Tbl
                                                 Tok2 = Filename
                                                 Tok3 = AttachParms->pPseudoColName */
#define DXERR_ATTACHMENT_NAME_TOO_LONG  15568 /* File attachment pathname/filename
                                                 too long
                                                 Tok1 = DBAL.CID.Tbl
                                                 Tok2 = Portion of full path/file name
                                                        that will fit in token
                                                        (and without the piece that
                                                        would take it over the limit)
                                                 Tok3 = AttachParms->pPseudoColName */
#define DXERR_WITHUR_NOT_ALLOWED        15569 /* WITH UR was requested, but is not allowed
                                                 on this table's database
                                                 Tok1 = DBAL.CID.Tbl             */
#define DXERR_DSSERR_ON_ARC_COLL        15570 /* And error occurred when attempting to add
                                                 the archive file to the archive colleciton.
                                                 Error info is in the DSS_AFC_ERROR structure
                                                 in DX_PARMS */

/* The following are errors reported in the DX_INPUT_TABLE structs
   when the DXCA reports a DXERR_TBL_VALIDATION_ERR.  If there is
   no corresponding DX_INPUT_TABLE, these errors are returned in
   the DXCA and Tok1 is DBAlias.CID.TableName and Tok2 contains
   the string that would have been pointed to by pTok in the
   DX_INPUT_TABLE struct.                                             */
#define DXTBLERR_TBL_AUTH             15601    /* ErrTok = DBMS UserID*/
#define DXTBLERR_COL_NOT_FOUND        15602    /* ErrTok = ColName    */
#define DXTBLERR_MISSING_PK           15603
#define DXTBLERR_INVALID_VIEW         15604
#define DXTBLERR_UNSUPPORTED_COL      15605    /* ErrTok = ColName    */
#define DXTBLERR_NO_ATTACHDIRS        15606    // pAttachDirs is not populated
                                               // Tok = AttachParms->pPseudoColName
#define DXTBLERR_NAMEPIECES_ERR       15607    // pNamePieces is not populated or it
                                               //   has an invalid Type value in it
                                               // Tok  = AttachParms->pPseudoColName
                                               // Tok2 = AttachParms->pNamePiece->pValue
#define DXTBLERR_NAMEPIECES_COL_ERR   15608    // pNamePieces has invalid col name
                                               //   or col has invalid type
                                               // Tok  = AttachParms->pPseudoColName
                                               // Tok2 = AttachParms->pNamePiece->pValue
#define DXTBLERR_TRIGGERCOL_ERR       15609    // pTriggerCol has invalid col name
                                               //   or col has invalid type
                                               // Tok  = AttachParms->pPseudoColName
                                               // Tok2 = AttachParms->pTriggerCol
#define DXTBLERR_ATTACHDIR_NOT_FOUND  15610    // Attachment directory does not exist
                                               // Tok  = AttachParms->pPseudoColName
                                               // Tok2 = AttachParms->pAttachDirs->pDirName
#define DXTBLERR_DUP_COLNAME          15611    // Pseudocolumn name is the same
                                               //  as an already existing col
                                               // Tok  = AttachParms->pPseudoColName
                                               // Tok2 = Pseudo Column Name
#define DXTBLERR_UNICODE_NONCONFORM   15612    // Table's unicode attribute does not
                                               // match connection's unicode attribute
                                               // and character columns are present in table
                                               // No error token


/* The following are errors reported in the DX_EXPLICIT_OBJECT_LIST
   structs when the DXCA reports a DXERR_OBJ_VALIDATION_ERR.          */
#define DXOBJERR_OBJ_NOT_FOUND        15651    /* DDL object not found*/


/* The following errors are used internally by Extract */
#define DXERR_ACTION_SKIP_ROW     15675  /* Action skipped row  */
#define DX_TABLES_ADDED           15676  /* Tables were added to a cell */
#define DX_PATH_IN_CELL           15677  /* a table in the path is already
                                            in the cell. */
#define DX_VALIDATION_ERROR       15678  /* DXValidateAD found a validation
                                            error. */
#define DX_STOP                   15679  /* An error has occurred to cause
                                           the Extract to stop.  Check the
                                           DXCB for reasons */
#define DX_KEY_HAS_NULL           15680  /* Key col found with NULL value */
#define DXWARN_TABLE_LIMIT        15681  /* Table reached its fetch limit */
#define DX_SCAN                   15682  /* A table scan is required      */

//RT: Begin, WI 4958: Number Ignore Option does not work with Oracle 10.2
#define DXERR_SRC_OUTOFRANGE      15683  /* The source value supplied is
                                            out of currently supported range*/
//RT: End, WI 4958: Number Ignore Option does not work with Oracle 10.2

/* The following return codes pertain to XFM_Open
   errors on the Input Extract File                                   */
#define  DXERR_INXF_INVALID_XF         15701
#define  DXERR_INXF_EXTRACT_IS_CONTROL 15702
#define  DXERR_INXF_OPENERR_XF         15703
#define  DXERR_INXF_INVALID_MODE       15704
#define  DXERR_INXF_INUSE_DS           15705
#define  DXERR_INXF_UNLICENSED_USAGE   15706
#define  DXERR_INXF_INVALID_CP         15707

/* The following codes are for XFM_Validate errors for the Input XF
   ErrTok1 will contain the Table Name
   ErrTok2 may contain XF Token2 which is a Column Name or ColMap Name
   ErrTok3 may contain AuxCode as the return code from a subordinate
           Component                                                 */
#define  DXERR_INXF_NO_COLS_MATCH             15751
#define  DXERR_INXF_NO_SOURCE_AND_NO_DEFAULT  15752
#define  DXERR_INXF_INVALID_CONVERSION        15753
#define  DXERR_INXF_TBL_NOT_REF_TBL           15754

/* XFM_Open security related errors */
#define  DXERR_PK_NOT_FOUND_SEC        15755 /*Val:Primary key not found-sec*/
#define  DXERR_REQ_COL_FILTERED_BY_SEC 15756 /*Val:Req col. filetered by sec*/
#define  DXERR_GUID_NOT_FOUND_FOR_SEC_FILE  15757
                                         /* XFM_Open: GUID not found in     */
                                         /*           archive directory for */
                                         /*           secured file          */

#define  DXERR_INVALID_SECURITY_TOKEN       15758
                                         /* XFM_Open: Invalid security token*/

#define  DXERR_FADS_NOT_FOUND_FOR_SEC_FILE  15759
                                         /* XFM_Open: FADS not found        */
                                         /*           for secured file      */
#define DXERR_REL_FILTERED_BY_SEC      15760
#define DXERR_REQ_TBL_FILTERED_BY_SEC  15761

#define DXERR_IDX_INTERNAL_ERROR       15762 /* Internal error building index */

/*--------------------------------------------------------------------*
 *  Conditional inclusion block for Error Message Text Array.         *
 *  The following structure is only included in the Process Services  *
 *  source module.                                                    *
 *--------------------------------------------------------------------*/
#ifdef DXX_ERR_MSG_BUILD
const static PST_CompErrMsg  DXXErrMsg[] =
{
 PST_COMP_ERR_MSG_ENTRY(DX_ERROR,"Extract error was detected")
 PST_COMP_ERR_MSG_ENTRY(DXERR_NO_TABLES,"No tables in the Input List")
 PST_COMP_ERR_MSG_ENTRY(DXERR_INVALID_START_TABLE,"Start table not in the "
            "input table list")
 PST_COMP_ERR_MSG_ENTRY(DXERR_REL_CHILD_TABLE_NOT_IN_LIST,"Relationship's "
            "child table is not in the input table list")
 PST_COMP_ERR_MSG_ENTRY(DXERR_REL_PAR_TABLE_NOT_IN_LIST,"Relationship's"
            "parent table is not in the input table list")
 PST_COMP_ERR_MSG_ENTRY(DXERR_DBALIAS_UNKNOWN,"Unknown DBAlias Name")
 PST_COMP_ERR_MSG_ENTRY(DXERR_TBL_NOT_FOUND,"Table not found")
 PST_COMP_ERR_MSG_ENTRY(DX_CANCEL,"Product asked for abort")
 PST_COMP_ERR_MSG_ENTRY(DXERR_PK_COL_NOT_FOUND,"PK Column not found in table")
 PST_COMP_ERR_MSG_ENTRY(DXERR_REL_NOT_FOUND,"Relationship not found")
 PST_COMP_ERR_MSG_ENTRY(DXERR_DUP_TABLES,"Duplicate tables or view base tables")
 PST_COMP_ERR_MSG_ENTRY(DXERR_INVALID_OVERRIDE_TBL,"Invalid table override")
 PST_COMP_ERR_MSG_ENTRY(DXERR_NOFM_COL_NOT_FOUND,"N or M column not found in table")
 PST_COMP_ERR_MSG_ENTRY(DXERR_NOFM_COL_TOO_LARGE,
                        "N or M column greater than 256 bytes")
 PST_COMP_ERR_MSG_ENTRY(DXERR_FILE_FULL,"Output Extract File Full")
 PST_COMP_ERR_MSG_ENTRY(DXERR_MAX_ROW_LIMIT_EXCEEDED,"User row limit exceeded")
 PST_COMP_ERR_MSG_ENTRY(DXERR_INVALID_XF,"Invalid Extract File")
 PST_COMP_ERR_MSG_ENTRY(DXERR_OPENERR_XF,"XFB couldn't open Extract File")
 PST_COMP_ERR_MSG_ENTRY(DXERR_DUP_PK,
         "XFB detected a duplicate PK value when building PK Index")
 PST_COMP_ERR_MSG_ENTRY(DXERR_DBERR,"Database error occured")
 PST_COMP_ERR_MSG_ENTRY(DXERR_INVALID_INPUT_PK,"Invalid input PK definition")
 PST_COMP_ERR_MSG_ENTRY(DXERR_REMOTE_TABLE,"Table or view is remote")
 PST_COMP_ERR_MSG_ENTRY(DXERR_INVALID_VIEW,"Underlying view tables have "
         "changed and the view cannot be resolved to base tables and coumns")
 PST_COMP_ERR_MSG_ENTRY(DXERR_REMOTE_TABLE_IN_REL,
         "Relationship references a remote table")
 PST_COMP_ERR_MSG_ENTRY(DXERR_TBL_VALIDATION_ERR,
         "One or more errors reported in DX_INPUT_TABLE(s)")
 PST_COMP_ERR_MSG_ENTRY(DXERR_OBJ_VALIDATION_ERR,
         "One or more errors reported in DX_EXPLICIT_OBJECT_LIST(s)")
 PST_COMP_ERR_MSG_ENTRY(DXERR_INVALID_REL,"Error in Relationship definition")
 PST_COMP_ERR_MSG_ENTRY(DXTBLERR_TBL_AUTH,"User not authorized to access table")
 PST_COMP_ERR_MSG_ENTRY(DXTBLERR_COL_NOT_FOUND,"Column not found in table")
 PST_COMP_ERR_MSG_ENTRY(DXTBLERR_MISSING_PK,"Table needs a PK")
 PST_COMP_ERR_MSG_ENTRY(DXTBLERR_INVALID_VIEW,"Underlying view table has "
         "changed and the view cannot be resolved to base tables and columns")
 PST_COMP_ERR_MSG_ENTRY(DXTBLERR_UNSUPPORTED_COL,
         "Table contains a column with an unsupported datatype")
 PST_COMP_ERR_MSG_ENTRY(DXOBJERR_OBJ_NOT_FOUND,"DDL object not found")
 PST_COMP_ERR_MSG_ENTRY(DXERR_VALIDATE_STMT_DBERR,
         "A DBMS Error occured while validating a Table's "
         "SQL Select Statement. Info is in the RDB Thrd CB. "
         "Tok1 may = DBAL.CID.Tbl")
 PST_COMP_ERR_MSG_ENTRY(DXERR_SORT_KEY_TOO_LARGE,"Sort Key > 256 bytes")
 PST_COMP_ERR_MSG_ENTRY(DXERR_SORT_COL_NOT_FOUND,
                        "Sort Col not found in Table Col List")
 PST_COMP_ERR_MSG_ENTRY(DXERR_INDEX_COL_UNKNOWN,"Index Column Unknown")
 PST_COMP_ERR_MSG_ENTRY(DXERR_INDEX_COL_INVALID,"Index Column Invalid")
 PST_COMP_ERR_MSG_ENTRY(DXERR_INDEX_TOO_LONG,"Index too long")
 PST_COMP_ERR_MSG_ENTRY(DXERR_RBA_FILE_FULL,"Extract Temp File Full")
 PST_COMP_ERR_MSG_ENTRY(DXERR_INVALID_HINT,
      "Invalid Optimization comment in the Where clause")


 PST_COMP_ERR_MSG_ENTRY(DXERR_BF_XTR_ACTION_SYNTAX,
                        "Invalid syntax in Before Extract Action")
 PST_COMP_ERR_MSG_ENTRY(DXERR_BF_XTR_ACTION_NOT_SUPPORTED,
                        "Action not supported in Before Extract Action")
 PST_COMP_ERR_MSG_ENTRY(DXERR_BF_XTR_ACTION_HV_NOT_FOUND,
                        "Invalid Host Variable in Before Extract Action")
 PST_COMP_ERR_MSG_ENTRY(DXERR_BF_XTR_ACTION_VAL_RDBERR,
                        "DBMS Validation error in Before Extract Action")
 PST_COMP_ERR_MSG_ENTRY(DXERR_AF_XTR_ACTION_SYNTAX,
                        "Invalid syntax in After Extract Action")
 PST_COMP_ERR_MSG_ENTRY(DXERR_AF_XTR_ACTION_NOT_SUPPORTED,
                        "Action not supported in After Extract Action")
 PST_COMP_ERR_MSG_ENTRY(DXERR_AF_XTR_ACTION_HV_NOT_FOUND,
                        "Invalid Host Variable in After Extract Action")
 PST_COMP_ERR_MSG_ENTRY(DXERR_AF_XTR_ACTION_VAL_RDBERR,
                        "DBMS Validation error in After Extract Action")
 PST_COMP_ERR_MSG_ENTRY(DXERR_BF_TBL_ACTION_SYNTAX,
                        "Invalid syntax in Before Table Action")
 PST_COMP_ERR_MSG_ENTRY(DXERR_BF_TBL_ACTION_NOT_SUPPORTED,
                        "Action not supported in Before Table Action")
 PST_COMP_ERR_MSG_ENTRY(DXERR_BF_TBL_ACTION_HV_NOT_FOUND,
                        "Invalid Host Variable in Before Table Action")
 PST_COMP_ERR_MSG_ENTRY(DXERR_BF_TBL_ACTION_VAL_RDBERR,
                        "DBMS Validation error in Before Table Action")
 PST_COMP_ERR_MSG_ENTRY(DXERR_AF_TBL_ACTION_SYNTAX,
                        "Invalid syntax in After Table Action")
 PST_COMP_ERR_MSG_ENTRY(DXERR_AF_TBL_ACTION_NOT_SUPPORTED,
                        "Action not supported in After Table Action")
 PST_COMP_ERR_MSG_ENTRY(DXERR_AF_TBL_ACTION_HV_NOT_FOUND,
                        "Invalid Host Variable in After Table Action")
 PST_COMP_ERR_MSG_ENTRY(DXERR_AF_TBL_ACTION_VAL_RDBERR,
                        "DBMS Validation error in After Table Action")
 PST_COMP_ERR_MSG_ENTRY(DXERR_BF_ROW_ACTION_SYNTAX,
                        "Invalid syntax in Before Row Action")
 PST_COMP_ERR_MSG_ENTRY(DXERR_BF_ROW_ACTION_NOT_SUPPORTED,
                        "Action not supported in Before Row Action")
 PST_COMP_ERR_MSG_ENTRY(DXERR_BF_ROW_ACTION_HV_NOT_FOUND,
                        "Invalid Host Variable in Before Row Action")
 PST_COMP_ERR_MSG_ENTRY(DXERR_BF_ROW_ACTION_VAL_RDBERR,
                        "DBMS Validation error in Before Row Action")
 PST_COMP_ERR_MSG_ENTRY(DXERR_ACTION_RDBERR,
                        "Archive Action resulted in a DBMS error")
 PST_COMP_ERR_MSG_ENTRY(DXERR_ACTION_ABORT,
                        "Archive Action notified Extract to Abort")

 PST_COMP_ERR_MSG_ENTRY(DXERR_IXFILE_OPEN_ERR,"Error openning Index File")
 PST_COMP_ERR_MSG_ENTRY(DXERR_TMPFILE_OPEN_ERR,
                        "Error openning Index Temp work file")
 PST_COMP_ERR_MSG_ENTRY(DXERR_IDX_TMPFILE_IOERROR,
                        "IO error on Index Temp File")
 PST_COMP_ERR_MSG_ENTRY(DXERR_IDX_TMPFILE_FULL,"Index Temp File Full")
 PST_COMP_ERR_MSG_ENTRY(DXERR_IDX_FILE_IOERROR,"IO error on Index File")
 PST_COMP_ERR_MSG_ENTRY(DXERR_IDX_FILE_FULL,"Index File Full")
 PST_COMP_ERR_MSG_ENTRY(DXERR_IDX_SORT_FAILURE,"Index Sort failure")
 PST_COMP_ERR_MSG_ENTRY(DXERR_INVALID_WHERE_CLAUSE,
                        "Invalid Table WHERE clause")
 PST_COMP_ERR_MSG_ENTRY(DXERR_ACTION_SKIP_ROW,
                        "Action indicated to skip row")
PST_COMP_ERR_MSG_ENTRY(DX_TABLES_ADDED,"Tables were added to a cell")
PST_COMP_ERR_MSG_ENTRY(DX_PATH_IN_CELL,
                         "A table in the path is already in the cell")
PST_COMP_ERR_MSG_ENTRY(DX_VALIDATION_ERROR,
                         "DXValidateAD found a validation error")
PST_COMP_ERR_MSG_ENTRY(DX_STOP,
                    "An error has occurred to cause the Extract to stop")
PST_COMP_ERR_MSG_ENTRY(DX_KEY_HAS_NULL,"Key col found with NULL value")
PST_COMP_ERR_MSG_ENTRY(DXWARN_TABLE_LIMIT,"Table reached its fetch limit")
PST_COMP_ERR_MSG_ENTRY(DX_SCAN,"A table scan is required")

PST_COMP_ERR_MSG_ENTRY(DXERR_PK_NOT_FOUND_SEC,
                       "Security validation: Primary key not found")
PST_COMP_ERR_MSG_ENTRY(DXERR_REQ_COL_FILTERED_BY_SEC,
                       "Security validation: Required column filtered")
PST_COMP_ERR_MSG_ENTRY(DXERR_GUID_NOT_FOUND_FOR_SEC_FILE,
                       "Security validation: GUID not found in archive directory for secured file")
PST_COMP_ERR_MSG_ENTRY(DXERR_INVALID_SECURITY_TOKEN,
                       "Security validation: Invalid security token")
PST_COMP_ERR_MSG_ENTRY(DXERR_FADS_NOT_FOUND_FOR_SEC_FILE,
                       "Security validation: FADS not found for secured file")
PST_COMP_ERR_MSG_ENTRY(DXERR_REL_FILTERED_BY_SEC,
                       "Security validation: Required relationship filtered")
PST_COMP_ERR_MSG_ENTRY(DXERR_REQ_TBL_FILTERED_BY_SEC,
                       "Security validation: Required table filtered")
PST_COMP_ERR_MSG_ENTRY(DXERR_IDX_INTERNAL_ERROR,
                       "Internal error occurred creating an archive Index")
 PST_COMP_ERR_MSG_END            /* Last entry is a null entry */

};
#endif





/* DX constants */
#define DX_CHILD_PATH       'C'
#define DX_PAR_PATH         'P'

/* Codes for the Notify Progress Exit */
#define DX_PROCESSING_IN_PROGRESS   'P'
#define DX_PROCESSING_DONE          'D'
#define DX_PROCESSING_MAYBE_DONE    'M'
#define DX_PROCESSING_WILL_BE_BACK  'B'
#define DX_PROCESSING_INDEX         'I'

/* Codes to indicate if Table row statistics are available */
#define DX_STATS_OK         1
#define DX_STATS_NOT_RUN    2
#define DX_STATS_UNAVAIL    3

/* Relationship usage constants */
#define  DX_REL_USED   0
#define  DX_REL_REF    1
#define  DX_REL_UNSEL  2

/* Other Miscellaneous defines */
#define DX_ERRTOK_LEN         160
#define DX_DATETIME_LEN        19
//KSB: 07/29/09: Support for Long Computer Name.
//#define DX_SERVER_NAME_LEN     15
#define DX_SERVER_NAME_LEN     OPTIM_MAX_COMPUTERNAME_LENGTH
#define DX_GROUP_NAME_LEN       8
#define DX_LONG_DESC_LEN      128
#define DX_INDEX_NAME_LEN      40

/* Pseudocolumn Name suffixes */
#define DX_PSEUDOCOL_SUFFIX_FILE   L"_FILE_NAME"
#define DX_PSEUDOCOL_SUFFIX_BLOB   L"_BLOB"
#define DX_PSEUDOCOL_SUFFIX_ATTRIB L"_ATTRIB"
#define DX_PSEUDOCOL_AFMF_NAME     L"AFMF_RRN"


/*--------------------------------------------------------------------*
 *  Active Compression parameter block                                *
 *--------------------------------------------------------------------*/
 typedef struct s_DXX_ACTIVE_COMPRESS
 {
    struct s_DXX_ACTIVE_COMPRESS * pNext; /* ptr to the next table threshold */

    /* Table Identification fields, Zero delimited strings            */
    wchar_t DBAliasName[RDB_DBALIAS_LEN+1];
    wchar_t CreatorID[RDB_CID_LEN+1];
    wchar_t TableName[RDB_TBL_NAME_LEN+1];

    long long inSize;           /* #bytes uncompressed */
    long long outSize;          /* #bytes compressed */
    short CompressThreshold;    /* User Given Table Threshold */
    short CompressionRatio;     /* Compression Percentage */
    UCHAR Reserved[44];         /* reserve space   */
 } DXX_ACTIVE_COMPRESS;

/* The following structure defines a linked list of columns.  It is used
   to identify the columns to extract for a table and identification
   of the Point and Shoot Columns.                                    */
/* If this list of structs is anchored by p1stSortKey in the main parm
   struct, then this list identifies the ordering for the rows on the
   extract file.  The order of the entries in the list are from major
   key to minor key.  The KeySeqNum field is ignored in this context */
typedef struct DXInputCol
{
    struct DXInputCol *       pNext;

    /* Column Identification - The following field points to a zero
       delimited string, but it does NOT need to be in the String Sink.
       The string is assumed to be in the proper case as per the
       referenced DBAlias's DBMS Catalog Case.                        */
    wchar_t *   pColName;

    /* If the UseInputPKDef in the DX_PARMS struct is TRUE, then this
       field is used to identify the table's Primary Key Columns.  If
       this field is > 0, then the number indicates this column is the
       nth column in the Primary Key for the associated table.  The
       first column in the Primary Key should have the value 1.  This
       field is ignored when the struct is being used to define Point
       and Shoot columns.                                             */
    short       KeySeqNum;

} DX_INPUT_COL;





/* The following structure defines a linked list of Archive Indexes
   to be built during the extract process.  Each table may have zero
   or more indexes.  The list is anchored in the Input Tbl struct.      */
typedef struct DXArchIdx
{
    struct DXArchIdx *  pNext;   /* Next Index for this table           */

    wchar_t           IndexName[DX_INDEX_NAME_LEN+1];  /* Index Name    */

    short             NumCols;       /* Num Cols in the following array */
    wchar_t *         pColName[16];  /* An array of Col Name pointers.
                                        Cols are in major to minor
                                        order.                          */
} DX_ARCHIVE_INDEX;




// DX_TBL_NAMEPIECE is a linked list of pieces that will be used to construct
// filenames for External File References
typedef struct sDXTblNamepiece
{
    struct sDXTblNamepiece * pNext;  // Next Entry
    UCHAR  Type;                     // Column or literal Indicator (see defines below)
    WCHAR *pValue;                   // Pointer to the colname or literal
} DX_TBL_NAMEPIECE;
// Codes for Type field in DX_TBL_NAMEPIECE
#define DXTBLNAMEPIECE_COLUMN   'C'   // Piece is a column name
#define DXTBLNAMEPIECE_LITERAL  'L'   // Piece is a literal value

// DX_TBL_ATTACH_DIRS is a linked list of directories where file attachments reside
typedef struct sDXTblAttachDirs
{
    struct sDXTblAttachDirs * pNext;    // Next Entry
    WCHAR                   * pDirName; // Pathname of directory
} DX_TBL_ATTACH_DIRS;

// DX_TBL_ATTACH_PARMS is a linked list of rules on how to handle file attachments
typedef struct sDXTBLAttachParms
{
    struct sDXTBLAttachParms * pNext; // Next Entry
    DX_TBL_ATTACH_DIRS * pAttachDirs; // List of directories to find attachments
    DX_TBL_NAMEPIECE   * pNamePieces; // Pieces of filenames of attachments
    WCHAR              * pTriggerCol; // Optional name of column that indicates
                                      //   whether row has an attachment
    UCHAR           StopWhenNotFound; // T/F - Stop when file not found
    WCHAR           * pPseudoColName; // Prefix for pseudo-column names
    UCHAR                 DeleteFile; // T/F - Delete file during DEL process
} DX_TBL_ATTACH_PARMS;




/* The following structure defines a linked list of tables that will
   participate in the Extract.  Each entry includes compile time and
   execution time attributes to further define the extract process.   */
typedef struct DXInputTbl
{
    struct DXInputTbl *       pNext;

    /* Table Identification - Each field points to a zero delimited
       string, but they do NOT need to be in the String Sink.  The
       CreatorID and TableName strings are assumed to be in the proper
       case as per the referenced DBAlias's DBMS Catalog Case.  If the
       DBMS referenced by the DBAlias does not require a CreatorID,
       the CreatorID field may be NULL.                               */
    wchar_t *   pDBAliasName;
    wchar_t *   pCreatorID;
    wchar_t *   pTableName;
    wchar_t *   pCorrName;  /* Optional Correlation Name to be used
                               with SQL for this Table                */

    UCHAR       RefTable;   /* Table is a reference table, TRUE | FALSE.
                               Ignored for DX_FUNC_EXEC calls         */
    UCHAR       OrderTable; /* Table is a reference table ot the
                               Start Table and the rows should be
                               be put to the XF in PK order.
                               If the table is the Start Table and
                               it involved in a cycle or has a non-PK
                               Point and Shoot List, rows will not
                               be in PK order.
                               Ignored for DX_FUNC_EXEC calls         */

    /* The following field is only required if a subset of the table's
       columns are to be extracted.  If this field is NULL, all columns
       for the table will be extracted.  This field is ignored for
       DX_FUNC_EXEC calls.                                            */
    DX_INPUT_COL *   pInputCols;

    /* The following field points to a list of structs that identify
       a sort key for the table on the Extract File                   */
    DX_INPUT_COL *   pSortCols;


    wchar_t *  pWhereClause;   /* Zero delimited string for the SQL
                                WHERE clause for this table.          */
    UCHAR      SelectWithUR;   /* T/F - If T then WITH UR clause will
                                be added to SELECT stmt               */

    /* If the func call is DX_FUNC_EXEC, then the following field
       is checked to see if the Compile Where clause is being
       overridded.  Valid values are:
             DX_OVERRIDE_NONE    - No override Where clause, use the
                                   one specified at compile time
             DX_OVERRIDE_PRESENT - Use this new Where Clause. If there
                                   is to be no Where Clause, set
                                   pWhereClause = NULL.               */
    UCHAR   WhereOverrideFlag;

    /* The following fields are input on DX_FUNC_COMPILE and
       DX_FUNC_COMPILE_AND_EXEC calls.                                */
    short   EveryNth;        /* Extract every nth row selected from
                                this table.  Zero indicates to extract
                                all rows selected.                    */
    long    RowLimit;        /* The maxmum number of extracted for
                                this table.  Zero indicates there is
                                no table limit.                       */
    /* If the func call is DX_FUNC_EXEC, then the following fields
       are checked to see if the Compile Values are being overriden.
       Valid values are:
             DX_OVERRIDE_NONE    - No override Where values, use
                                   compile value
             DX_OVERRIDE_PRESENT - Use this new value                 */
    UCHAR   EveryNthOverrideFlag;
    UCHAR   RowLimitOverrideFlag;

    /* The following field indicates if the table is logically traversed
       bas-ed on the input table and rel definitions.  It is set by
       Extract during DX_FUNC_SHOW_STEPS, DX_FUNC_VALIDATE,
       DX_FUNC_COMPILE, and DX_FUNC_COMPILE_AND_EXEC calls.           */
    UCHAR   TableTraversed;        /* TRUE or FALSE                   */

    /* The following field indicates whether a table has Catalog
       Statistics to help determine the access strategy.  It is set
       by Extract during DX_FUNC_VALIDATE, DX_FUNC_COMPILE, and
       DX_FUNC_COMPILE_AND_EXEC calls.                                */
    UCHAR   TblStatsAvail;   /* DX_STATS_OK      - Stats were there
                                DX_STATS_NOT_RUN - Stats were NOT run
                                DX_STATS_UNAVAIL - DBMS doesn't support
                                                   statistics         */

    /* When the DXCA indicates a DXERR_TBL_VALIDATION_ERR, one or more
       DX_INPUT_TABLE structs are filled in with a Table Error Code.
       The codes are the DXTBLERR_... constants defined above.        */
    short   ErrCode;
    wchar_t ErrTok[DX_ERRTOK_LEN+1];  /* ColName or User ID, check the
                                         DXTBLERR_.. constants for
                                         details.                     */
    wchar_t ErrTok2[DX_ERRTOK_LEN+1]; /* Additional Error Info        */
    wchar_t ErrTok3[DX_ERRTOK_LEN+1]; /* Additional Error Info        */

    DX_ARCHIVE_INDEX *   p1stArchiveIndex;   /* Ptr to first archive
                                                index definition for
                                                this table.           */

    wchar_t *pBefTblAct;        /* Ptr to Before Table Action Statement.
                                   NULL if none.                        */
    wchar_t *pBefTblActDBAliasName;  /* DBAlias for Before Table Action */
    UCHAR    BefTblActErrControl;    /* DX_ACTION_ERR_STOP, ...         */
    UCHAR    BefTblActHVChar;        /* Host Var Char for this action   */

    wchar_t *pAftTblAct;        /* Ptr to After Table Action Statement.
                                   NULL if none.                        */
    wchar_t *pAftTblActDBAliasName;  /* DBAlias for After Table Action  */
    UCHAR    AftTblActErrControl;    /* DX_ACTION_ERR_STOP, ...         */
    UCHAR    AftTblActHVChar;        /* Host Var Char for this action   */

    wchar_t *pBefRowAct;        /* Ptr to Before Row Action Statement.
                                   NULL if none.                        */
    wchar_t *pBefRowActDBAliasName;  /* DBAlias for Before Row Action   */
    UCHAR    BefRowActErrControl;    /* DX_ACTION_ERR_STOP, ...         */
    UCHAR    BefRowActHVChar;        /* Host Var Char for this action   */

    DX_TBL_ATTACH_PARMS *pAttachParms; /* Parms for file attachments    */
} DX_INPUT_TABLE;



/* Some defines for the following struct */
#define  DX_PATH_INDEX_FULL      'F'
#define  DX_PATH_INDEX_PARTIAL   'P'
#define  DX_PATH_INDEX_NONE      'N'

//RT: Ignore Empty Key
/* Structure that stores the ignore options for each
   column in the ralationship. It is defined as a chain
   of elements and is used in DX_INPUT_REL and DX_REL
   structures
*/
typedef struct DXRelIgnOptions
{
   struct   DXRelIgnOptions *pNext; /* Pointer to the next entry */
   short    sIgnoreOptions;         /* Ignore Options */
   short    sCriteria;              /* Criteria AND/OR */
   long     lNumber;                /* Ignore Number */
   wchar_t  pszColName[1];          /* Column Name */
} DX_REL_IGN_OPTIONS;
//RT: Ignore Empty Key
/* The following structure defines a linked list of relationships that
   will participate in the Extract.  Currently, this structure is only
   used at Extract Compile time.                                      */
typedef struct DXInputRel
{
    struct DXInputRel *  pNext;

    UCHAR       Type;    /* Relationship Type,  RDB_DBMS, RDB_PST or
                            RDB_GENERIC.                              */

    UCHAR       Usage;   /* DX_REL_USED    Rel to be used in extract
                            DX_REL_REF     Rel to a Reference table
                            DX_REL_UNSEL   Rel is unselected          */

    /* Rel Identification - Each field points to a zero delimited
       string, but they do NOT need to be in the String Sink.  The
       CreatorID, TableName and RelName strings are assumed to be in the
       proper case for the referenced DBAlias's DBMS Catalog.  If the
       relationship is RDB_GENERIC, then the CreatorID fields should
       still point to the Parent and Child Table's Creator IDs.  If the
       DBMS referenced by the DBAlias does not require a CreatorID,
       the CreatorID field may be NULL.                               */
    /* These next three names identify the Child Table or View as
       identified in the DX_INPUT_TABLE List                          */
    wchar_t *   pChlDBAliasName;
    wchar_t *   pChlCreatorID;
    wchar_t *   pChlTableName;
    /* These next three names identify the Parent Table or View as
       identified in the DX_INPUT_TABLE List                          */
    wchar_t *   pParDBAliasName;
    wchar_t *   pParCreatorID;
    wchar_t *   pParTableName;
    /* These next four names identify the Relationship Name.  If the
       above Child names identify a table, then the following
       DBAliasName, CreatorID, and TableName are the same as
       pChlDBAliasName, pChlCreatorID, and pChlTableName.  However, if
       the above Child names identify a view, then the following names
       are the true relationship name as defined to the view's base
       table.                                                         */
    wchar_t *   pRelDBAliasName;
    wchar_t *   pRelCreatorID;
    wchar_t *   pRelTableName;
    wchar_t *   pRelName;

    UCHAR   GetParents;          /* Question 1: Follow the parent path
                                    for this rel when a child row is
                                    is selected, TRUE | FALSE         */
    UCHAR   AlwaysGetChildren;   /* Question 2: Follow the child path
                                    for this rel, even when the par row
                                    row is selected via a parent path,
                                    TRUE | FALSE.                     */
    short   LimitNumChlPerPar;   /* Limit the number of children rows
                                    selected per parent row.
                                    -1 : Select no children
                                     0 : No Limit
                                     n : Only n children per parent   */

    UCHAR   ForceParTblAccess;   /* Directives to force how Extract will
                                    access Parent rows related to
                                    Extracted Child rows for this rel:
                                        DX_FORCE_NONE   - Don't force
                                        DX_FORCE_SCAN   - Force Tbl Scan
                                        DX_FORCE_KEYLOOKUP - Force
                                              Select via PK columns   */

    UCHAR   ForceChlTblAccess;   /* Directives to force how Extract will
                                    access Child rows related to
                                    Extracted Parent rows for this rel:
                                        DX_FORCE_NONE   - Don't force
                                        DX_FORCE_SCAN   - Force Tbl Scan
                                        DX_FORCE_KEYLOOKUP - Force
                                              Select via FK columns   */

    /* The following field indicates if the rel is logically traversed
       bas-ed on the input table and rel definitions.  It is set by
       Extract during DX_FUNC_SHOW_STEPS, DX_FUNC_VALIDATE,
       DX_FUNC_COMPILE, and DX_FUNC_COMPILE_AND_EXEC calls.           */
    UCHAR   RelTraversed;               /* Rel will be traversed during
                                           the extract, T|F.          */
    void *  IntHandle;                  /* Internal field used by
                                           Extract.  This field will
                                           change during extract.     */

    UCHAR    ParPathUsed;       /* TRUE|FALSE the path from the dependent
                                   to the parent is used for this
                                   extract traversal.                 */
    UCHAR    ParPathIndex;      /* One of the following codes for this
                                   path:
                                     DX_PATH_INDEX_FULL - there is a
                                             full index for this path.
                                     DX_PATH_INDEX_PARTIAL there is a
                                             partial for this path.
                                     DX_PATH_INDEX_NONE - there is no
                                             index for this path.     */

    UCHAR    DepPathUsed;       /* TRUE|FALSE the path from the parent
                                   to the dependent is used for this
                                   extract traversal.                 */
    UCHAR    DepPathIndex;      /* Same codes as ParPatnIndex.        */

    short    ParCompoundKeyLookupCnt;  /* Number of key lookups to
                                   generate into the Where Clause
                                   for Parent Table Lookups.
                                   0 indicates to use the default.    */
    short    DepCompoundKeyLookupCnt;  /* Number of key lookups to
                                   generate into the Where Clause
                                   for Dependent Table Lookups.
                                   0 indicates to use the default.    */

    UCHAR    Resv[8];           /* Reserved space                     */

    //RT: Ignore Empty Key
    DX_REL_IGN_OPTIONS  *IgnoreOptions;  /* Bit masked variable
                                             bit 1 -> Null
                                             bit 2 -> Blank
                                             bit 3 -> Zero Length
                                             bit 4 -> Number
                                          */
    //RT: Ignore Empty Key
} DX_INPUT_REL;








/* Structure that defines a single entry in the Point and Shoot key
   values list    */
typedef struct DXPNSKeyValue {
  struct   DXPNSKeyValue *   pNext;   /* Pointer to the next entry */
  UCHAR    KeyValue[1];               /* Char block of values      */
} DX_PNS_KEY_VALUE;

/* Macro that returns the size of one value_entry base on the size */
/* of the key values.                                              */
#define DX_PNS_KEY_VALUE_SIZE(n)  ((sizeof(struct DXPNSKeyValue)) +    \
                         ((n) - 1))


/* The following struct defines a Point and Shoot List, the values
   and the their field attributes.                                  */
typedef struct DXPNSList {
  long                 NumValues;    /* Number of values in the list  */
  RDB_COL_LIST *       pColList;     /* Key Field definitions         */
  DX_PNS_KEY_VALUE *   pKeyValues;   /* Ptr to list of Key Values     */
} DX_PNS_LIST;




/* Note: The structures below are basically copies of the structures
         currently in use on the MVS product.  The only change is to
         relpace pointers to AD Table and Rel structs to the
         corresponding DX_INPUT_TABLE and REL structs.                */


/* This structure is used to return the results of an Extract.  It is
   a linked list of Table entries and the number of rows extracted
   for each. */
typedef struct s_DXOutputTable
{
    struct s_DXOutputTable *   pNext;  /* ptr to the next tbl struct */

    /* Table Identification fields, Zero delimited strings            */
    wchar_t DBAliasName[RDB_DBALIAS_LEN+1];
    wchar_t CreatorID[RDB_CID_LEN+1];
    wchar_t TableName[RDB_TBL_NAME_LEN+1];

    long    NumRowsExtracted;           /* Num rows extracted */
    long    NumStartTable1stPassRows;   /* 0 if not a Start Table */
    long    RowFailCount;               /* Number of rows we tried to
                                           read but couldn't because
                                           the BLOBs and CLOBs are too
                                           long.                      */
    long    NumDBMSXlateErrs;           /* Num of times a row failed
                                           because of a DBMS data
                                           translation error          */
    /* The following are statistics about how the table was accesses
       and the elapsed time to process the table.                     */
    long    NumTblScans;      /* Num times tbl was processed via Scan */
    long    NumTblKeyLookups; /* Num times processed via Key Lookup   */
    float   ScanTime;         /* Elapsed time in secs to do Scans     */
    float   KeyLookupTime;    /* Elapsed time in sec to do Key Lookups*/
    //KSB: 25/05/2011 - Teradata Native support
    short   TableAccessConnMode;     /* For now,used for Teradata only -
                                        Connection mode. CLI or TPT for table
                                        access.  */
} DX_OUTPUT_TABLE;


/* The following structure defines a linked list of relationships for
   which information must be reported to our caller   */
typedef struct DXOutputRel
{
    struct DXOutputRel *  pNext;

    wchar_t *   pChlDBAliasName;   // ID Information.  These pointers contain
    wchar_t *   pChlCreatorID;     // the same values as their counterparts in
    wchar_t *   pChlTableName;     // DX_INPUT_REL.
    wchar_t *   pParDBAliasName;   // *
    wchar_t *   pParCreatorID;     // *
    wchar_t *   pParTableName;     // *
    wchar_t *   pRelDBAliasName;   // *
    wchar_t *   pRelCreatorID;     // *
    wchar_t *   pRelTableName;     // *
    wchar_t *   pRelName;          // *
    short       RevisedParCompoundKeyCount; /* If Compound Key lookup value for Parent was
                                               automatically reduced due to DBMS
                                               limitations, the new value will be
                                               in this field.  Otherwise this field
                                               will be zero */
    short       RevisedDepCompoundKeyCount; /* If Compound Key lookup value for Dependent was
                                               automatically reduced due to DBMS
                                               limitations, the new value will be
                                               in this field.  Otherwise this field
                                               will be zero */
    UCHAR       Resv[20];          // Spare
} DX_OUTPUT_REL;









/* These structures are returned when the function is DX_FUNC_SHOW_STEPS.
   They define the table traversal strategy used to perform the
   extract. */
/* Each traversal step will identify the one or more tables visited
   in that step.  Multiple tables are visited for cycle cells.  This
   list is anchored from the corresponding step entry.  There is also
   a list of untraversed tables anchored in DXPL_MAIN. */
typedef struct s_DXPLTable
{
    struct s_DXPLTable *   pNext;       /* ptr to the next tbl struct */
    DX_INPUT_TABLE *       pInputTbl;   /* Ptr to the corresponding
                                           input TBL structure.  This
                                           may be NULL when the Start
                                           Table is in a cycle.       */

} DXPL_TABLE;


/* Each traversal step will identify the zero or more cycle paths to
   be traversed between the tables in the cell to be processed in that
   step.  Only cycle cells have this list.  This list is anchored from
   the corresponding step entry. */
typedef struct s_DXPLCyclePath
{
    struct s_DXPLCyclePath *   pNext;   /* ptr to the next path struct.
                                           This list is anchored in the
                                           corr. DXPLStep entry. */
    DX_INPUT_REL *             pInputRel; /* Ptr to the corresponding
                                             input REL structure      */
} DXPL_CYCLE_PATH;


/* This struct is used for a list of relationships not followed in this
   traversal.  It is anchored in DXPL_MAIN. */
typedef struct s_DXPLUnTravRel
{
    struct s_DXPLUnTravRel *   pNext;   /* ptr to the next rel struct.
                                           This list is anchored in the
                                           DXPL_MAIN. */
    DX_INPUT_REL *             pInputRel; /* Ptr to the corresponding
                                           input REL structure        */
} DXPL_UNTRAV_REL;


/* The following struct is used for two purposes.  The first is a linked
   paths that are to be followed after processing a step.  This means
   that PKs or FKs from rows selected in this step will be used to
   select parent or child rows in a subsequent step.  Each entry
   identifies the relationship to be followed, in the parent or child
   direction, and the step the saved PKs or FKs will be used.

   The other purpose is the reverse.  It identifies the previous steps
   that selected related rows that control the rows selected in this
   step. */
typedef struct s_DXPLTravPath
{
    struct s_DXPLTravPath *   pNext;  /* ptr to the next path struct.
                                         This list is anchored in the
                                         corr. DXPLStep entry.        */
    DX_INPUT_REL *   pInputRel;       /* Ptr to the corresponding
                                         input REL structure.  This
                                         may be NULL if the Start tbl
                                         is in a cycle.               */
    UCHAR            PathType;        /* DX_CHILD_PATH | DX_PAR_PATH  */
    short            StepNum;         /* For Exit paths, the step number
                                         the related rows will be
                                         selected.  For Entry paths, the
                                         step the related rows were
                                         selected.                     */
} DXPL_TRAV_PATH;


/* this struct defines the traversal step.  It is mainly an anchor
   for the tables, cycle paths, and entry/exit paths for this step. */
typedef struct s_DXPLStep
{
    struct s_DXPLStep *   pNext;      /* ptr to the next step struct. */
    short   StepNum;            /* Ordinal step number */
    short   CycleCell;          /* This step is for a cycle cell, T|F */
    short   NumTbls;            /* Num of tables in the list */
    short   NumCyclePaths;      /* Num of cycle paths in the list */
    short   NumEntryPaths;      /* Num of entry paths in the list */
    short   NumExitPaths;       /* Num of exit paths in the list */
    DXPL_TABLE *        p1stTbl;         /* ptr to the 1st table */
    DXPL_CYCLE_PATH *   p1stCyclePath;   /* ptr to the 1st Cycle path */
    DXPL_TRAV_PATH *    p1stEntryPath;   /* ptr to the 1st Entry Path */
    DXPL_TRAV_PATH *    p1stExitPath;    /* ptr to the 1st Exit Path */
} DXPL_STEP;


/* this struct anchors the explain structures. */
typedef struct s_DXPLMain
{
    short   NumSteps;           /* The number of steps in the list */
    short   NumUnTravTbls;      /* Num of tables not traversed */
    short   NumUnTravRels;      /* Num of relationships not traversed */
    DXPL_STEP *          p1stStep;        /* ptr to the 1st table */
    DXPL_TABLE *         p1stUnTravTbl;   /* Ptr to the 1st untraversed
                                             table entry */
    DXPL_UNTRAV_REL *    p1stUnTravRel;   /* Ptr to the 1st untraversed
                                             relationship entry */
} DXPL_MAIN;






/*--------------------------------------------------------------------*
    Structure: DX_DDL_OBJECT_TYPE
    The following structure contains byte-flags that enumerate the
    objects to be included in an extract file.  These byte-flags
    will contain a 1 (e.g. TRUE-value) when the object is to be
    included and a 0 (e.g. FALSE-value) when the object is NOT to
    be included.

    Following the Byte Flags are a set of corresponding counters for
    the number of each object type that was extracted.  These counters
    are set by DDL Extract.
 *--------------------------------------------------------------------*/
typedef struct s_DX_DDLObjectType
{
    UCHAR                   ucExtendedTableAttr;
    UCHAR                   ucPrimaryKey;
    UCHAR                   ucRelationship;
    UCHAR                   ucIndex;
    UCHAR                   ucFieldProcs;
    UCHAR                   ucTrigger;
    UCHAR                   ucView;
    UCHAR                   ucSynonymAlias;
    UCHAR                   ucFunction;
    UCHAR                   ucProcedure;
    UCHAR                   ucPackage;
    UCHAR                   ucSequence;
    UCHAR                   ucUDT;
    UCHAR                   ucDefault;
    UCHAR                   ucRule;
    UCHAR                   ucAssembly;
    UCHAR                   ucPartitionFunction;
    UCHAR                   ucPartitionScheme;
    UCHAR                   ucReserved[46];

    long                    NumExtractedTables;
    long                    NumExtractedPKS;
    long                    NumExtractedRelatedRels;
    long                    NumExtractedRelatedIndexes;
    long                    NumExtractedFieldProcs;
    long                    NumExtractedTriggers;
    long                    NumExtractedViews;
    long                    NumExtractedSynonymsAliases;
    long                    NumExtractedFunctions;
    long                    NumExtractedProcedures;
    long                    NumExtractedPackages;
    long                    NumExtractedSequences;
    long                    NumExtractedUDTs;
    long                    NumExtractedDefaults;
    long                    NumExtractedRules;
    long                    NumExtractedAssemblies;
    long                    NumExtractedPartitionFunctions;
    long                    NumExtractedPartitionSchemes;
    long                    NumExtractedSecComs;
    long                    NumExtractedSecPols;
    long                    NumExtractedSecLabs;
    long                    NumExtractedReserved[43];
} DX_DDL_OBJECT_TYPE;




/*--------------------------------------------------------------------*
    Structure: DX_EXPLICIT_OBJECT_LIST
    The following structure contains information defining explicit
    DDL Objects to Extract
 *--------------------------------------------------------------------*/
typedef struct s_DX_ExplicitObjectList
{
    struct s_DX_ExplicitObjectList *pNext;
                            /* Pointer to the next entry              */
    wchar_t                         *pDBAlias;
                            /* Pointer to the DBAlias                 */
    wchar_t                         *pCreator;
                            /* Pointer to the creator-id              */
    wchar_t                         *pName;
                            /* Pointer to the table-name              */
    short                            ObjType;
                            /* Object Type, one of:                   */
                            /* DDL_OBJ_VIEW                           */
                            /* DDL_OBJ_SYNONYM                        */
                            /* DDL_OBJ_ALIAS                          */
                            /* DDL_OBJ_SEQUENCE                       */
                            /* DDL_OBJ_FUNCTION                       */
                            /* DDL_OBJ_PROCEDURE                      */
                            /* DDL_OBJ_PACKAGE                        */
                            /* DDL_OBJ_DATATYPE                       */
                            /* DDL_OBJ_RULE                           */
                            /* DDL_OBJ_DEFAULT                        */

    /* The following is field used to return info to the caller       */
    /* When the DXCA indicates a DXERR_OBJ_VALIDATION_ERR, one or more
       DX_EXPLICIT_OBJECT_LIST structs are filled in with an Obj Error
       Code. The codes are the DXOBJERR_... constants defined above.  */
    short   ErrCode;
} DX_EXPLICIT_OBJECT_LIST;



/* The following struct is primary prameter block passed to the
   Extract Log Exit.  Various fields are filled in depending on the
   Log Code.  See the Log Codes for a spec on which fields are filled
   if for each code.                                                 */
typedef struct s_DXLogCB
{
    short   LogCode;                    /* See constants below        */
    void *  pProdAnchor;

    /* The following table identification fields are always in        */
    /* Table Identification fields, Zero delimited strings            */
    wchar_t * pDBAliasName;
    wchar_t * pCreatorID;
    wchar_t * pTableName;

    RDB_COL_LIST *   pRowCols;   /* Row's Col Definitions */
    UCHAR *          pRowData;   /* The Row in error      */

} DX_LOG_CB;

/* Log Codes */
/* The following Log Codes have all the fields filled in the
   APL_LOG_CB struct.                                                 */
#define DXLOG_DUP_PK              1     /* XFB detected a duplicate
                                           Primary Key value          */


/* DX_STATS_REL is a linked list of relationships that provides input
   keys to a table.  How many and which keys will determine how the
   table is accessed.                                                */
typedef struct sDXStatsRel
{
    struct sDXStatsRel *  pNext;   /* Next Rel Struct for this table step */

    /* These next four names identify the Relationship Name. */
    /* If a parent direction, these fields are NULL */
    wchar_t *   pRelDBAliasName;
    wchar_t *   pRelCreatorID;
    wchar_t *   pRelTableName;
    wchar_t *   pRelName;

    long        NumKeysToLookup;  /* Number of key values to lookup */

    UCHAR       Direction;        /* Direction relationship is being
                                     traversed:   'P' - Parent
                                                  'D' - Dependent   */
    UCHAR       fIndexed;         /* TRUE|FALSE, rel has a supporting index */
    UCHAR       ForcedAccess;     /* Directives to force Extract to scan
                                     or use Key Lookup.
                                     DX_FORCE_NONE   - Don't force
                                     DX_FORCE_SCAN   - Force Tbl Scan
                                     DX_FORCE_KEYLOOKUP - Force Key Lookup
                                              Select via PK columns   */

    UCHAR       Filler1;
    long        CompoundKeyCount;  /* Number of keys values to provide
                                      with each Open cursor            */
    wchar_t *   pKeyLookupSQL;     /* Ptr to ZDlm string of the SQL
                                      WHERE clause text to lookup one
                                      key value. */
    long        KeyLength;         /* Key length in bytes.               */

    float       DB2KeyLookupCost;  /* is 0.0 in not applicable           */
    float       DB2ScanCost;  /* is 0.0 in not applicable                */
} DX_STATS_REL;



/* DX_STATS_DBMS_ACCESS is a linked list of structs that provides detailed
   information about the access to the related table.  FOr a given table
   step there may be multiple DBMS Access structs if there were multiple
   key lookups to perform.                                             */
typedef struct sDXStatsDBMSAccess
{
    struct sDXStatsDBMSAccess *  pNext;   /* Next DBMS Access Struct
                                             for this table step       */

    short   AccessType;    /* How the table is being accessed:
                                'S' - Single Cursor scan
                                'P' - PK lookup
                                'F' - FK lookup                       */
    long   CompoundKeyCount;  /* If a PK or FK Lookup, the number of
                                 keys values to provide with each
                                 Open cursor                          */

    wchar_t *  pRelName;   /* If a FK Lookup, the relationship name
                              of the FK.                              */

    long     NumOpenCursors;     /* Number of Open Cursors executed   */
    long     NumRowsFetched;     /* Number of rows fetched            */
    long     NumRowsWrittenToXF; /* Number of rows written to the XF  */
    long     ElapsedTime;        /* Max elapsed time in seconds recorded
                                    by any of the threads working on
                                    the relationship                  */
    long     TimeInDBMSCalls;    /* Elapsed time in seconds for the
                                    calls to the DBMS, for all threads*/
    long     TimeInOtherCalls;   /* Elapsed time in seconds for any
                                    other activities for all threads  */
    long     TimeInRowArchiveAct;/* Elapsed time in seconds for
                                    BeforeRowExtract archive actions  */
    void    *pStructID;          /* Unique identifier, internal use   */
    // KSB: 25/05/2011 - Teradata native support
    // To calculate the average data size extracted per second.
    unsigned long long     AverageTblDataSize;
                                 /* For now, Teradata only. in Kilo bytes */
} DX_STATS_DBMS_ACCESS;




/* A linked list of the following structs are optionally returned
   to the caller to provide info about the strategies used and the
   timings during an extract. They are a linked list of structs that
   are put to the end of the list as extract steps occur.            */

typedef struct sDXStatsStep
{
    struct sDXStatsStep *  pNext;   /* Next Step Struct */

    DX_STATS_REL *         pInputRels;  /* Linked list of rels that
                                           provide input keys for
                                           this table */
    DX_STATS_DBMS_ACCESS *  pDBMSAccess; /* Linked list of detailed
                                            info about the DBMS access */

    /* Table Identification fields, Zero delimited strings  */
    wchar_t * pDBAliasName;
    wchar_t * pCreatorID;
    wchar_t * pTableName;

    /* DBMS Type and version */
    UCHAR      DBMSType;       /* RDB DBMS Type code.
                                     RDB_ORACLE, RDB_SYBASE, ...      */
    UCHAR *    pDBMSVerStr;    /* Ptr to DBMS Version String as
                                  returned from the DBMS. Ptr to
                                  a NULL string if unavailible        */

    /* The step number as per Show Steps */
    long      StepNum;         /* -1 indicates a Reference Table */

    long      RowLen;          /* Row length */
    short     NumTblCols;      /* Number of cols in the Table */
    UCHAR     fInACycle;       /* TRUE|FALSE, table in an extract cycle */
    UCHAR     fTableHasLOBs;   /* TRUE|FALSE, table has LOB Columns */

    long      NumTblRows;      /* Number of rows in the table, -1 if
                                  unavailible                       */
    wchar_t * pUserWhereClause; /* Ptr to ZDlm string of the WHERE
                                   clause specified by the user.
                                   NULL if none was specified.      */

    short     fStrategyInfoAvail;  /* TRUE or FALSE, the following
                                      strategy information is availible */
    /* Start Strategy Info ***************************************** */
    UCHAR     ParStrategy;     /* Parent Path Strategy, see list below */
    UCHAR     DepStrategy;     /* Dependent Path Strategy, see list below */
    /* The following 4 longs indicate how many keys of vsrious types
       need to be looked up                                          */
    long      NumPKsWithIndex;   /* Num PKs with supporting index    */
    long      NumPKsWoIndex;     /* Num PKs without supporting index */
    long      NumFKsWithIndex;   /* Num FKs with supporting index    */
    long      NumFKsWoIndex;     /* Num FKs without supporting index */
    /* End Strategy Info ******************************************* */

    short     NumThreads;   /* Num of threads used to perform the extract */
    UCHAR     SelectWithUR; /* T/F - WITH UR added to SELECT stmt         */
    long      ExtractMaxAllocation; // This will be maximum total number of bytes to allocate for all buffers when running an extract.  Allowed values will be 64k to 1024k  (where k=1024)
    UCHAR     FileComp;
    //KSB: 25/05/2011 - Teradata Native support
    short     TableAccessConnMode;     /* For now,used for Teradata only -
                                        Connection mode. CLI or TPT for table
                                        access.  */
    WCHAR*    pAccountID;   /* Teradata account ID */
    UCHAR*    pCharSetName; /* Character set name */
} DX_STATS_STEP;


/* Defines for Access strategy */
#define DXX_STRATEGY_NO_KEYS          1   // No Keys to Lookup
#define DXX_STRATEGY_ONLY_1_KEY       2   // Only one Key value to Lookup
#define DXX_STRATEGY_FORCE_SCAN       3   // User Forced Scan
#define DXX_STRATEGY_FORCE_LOOKUP     4   // User Forced Key Lookup
#define DXX_STRATEGY_LOOKUP           5   // KeyLookup - All Keys indexed
#define DXX_STRATEGY_SCAN_NO_INDEX    6   // Scan - No Index
#define DXX_STRATEGY_SCAN_TOO_MANY_KEYS 7 // Scan - Too many keys to lookup
#define DXX_STRATEGY_LOOKUP_TOO_FEW_KEYS 8 //KeyLookup - Too few keys to scan



// DX_ARC_COLLECTION is a linked list of Archive Collection names.
// The archive produced by DXX will be added to each of the named Collections.
typedef struct sDXArcCollection
{
    struct sDXArcCollection * pNext; // Next Entry
    WCHAR *pID;                      // Pointer to the collection ID
    WCHAR *pName;                    // Pointer to the collection name
} DX_ARC_COLLECTION;



/* The following structure defines the primary Parameter Block passed
   to Extract.                                                        */
typedef struct DXParms
{
    /* The next two fields define the Default DBAlias Name and Default
       Creator ID.  They are merely copied into the Extract File.  They
       are not used in any other way by the Extract Process.          */
    wchar_t *   pDefDBAliasName;
    wchar_t *   pDefCreatorID;

    /* The following field points to a linked list of the tables that
       participate in the Extract.  It is required for the
       DX_FUNC_SHOW_STEPS, DX_FUNC_VALIDATE, DX_FUNC_COMPILE, and the
       DX_FUNC_COMPILE_AND_EXEC calls.  It is optional for DX_FUNC_EXEC
       calls where it provides overrides values for selected tables.
       For DX_FUNC_EXEC a partial list may be passed, only for those
       tables needing overrides.                                      */
    DX_INPUT_TABLE *    pTables;

    /* The following field points to a linked list of relationships
       that participate in the Extract. It is required for the
       DX_FUNC_SHOW_STEPS, DX_FUNC_VALIDATE, DX_FUNC_COMPILE, and the
       DX_FUNC_COMPILE_AND_EXEC calls.  It is ignored by the others.
       The caller should have done standard AD MergeRels processing
       before calling extract.  Extract will no longer duplicate that
       work and will not report on new or removed relationships.      */
    /* It is assumed that all Rels on this list have a Usage or
       DX_REL_USED.                                                   */
    DX_INPUT_REL *     pRels;

    /* The following field points to a linked list of relationships
       that do NOT participate in the Extract, but are to be added
       to the relationship definitions on the extract file.  These
       might relationships to Reference Tables or relationships the
       user has UnSelected as participating in the extract.
       It is assumed that all rels on this list have a Usage of
       DX_REL_REF or DX_REL_UNSEL.                                    */
    DX_INPUT_REL *     pAuxRels;

    UCHAR     UseInputPKDef;   /* If TRUE, then Catalog Services is NOT
                                  called to obtain PK information about
                                  tables.  Instead the KeySeqNum fields
                                  in the DX_INPUT_COL structs are used
                                  to determine the Table's PK columns.*/

    /* General Parameters *********************************************/
    UCHAR   ValidateSQL;    /* TRUE | FALSE
                               For DX_FUNC_VALIDATE or DX_FUNC_COMPILE
                               calls, Prepare the SQL Select statements
                               as part of the validation process.
                               If a DX_FUNC_COMPILE_AND_EXEC or
                               DX_FUNC_EXEC call, prepare all the SQL
                               before starting the extract process.   */
    UCHAR   CountDBMSXlateErrs;  /* TRUE | FALSE, count the number of
                                    times for each table a row fails
                                    because of DBMS data translation
                                    errors                            */
    UCHAR   XtractFromXF;  /* TRUE | FALSE, extract source is an
                              file.                                   */

    /* Compile Parameters *********************************************/
    /* These parameters need not be filled in for DX_FUNC_EXEC calls.
       Some of them may be overridden at execute time.                */

    DX_INPUT_TABLE *    pStartTable;  /* Pointer to a table in the
                                         pTables list for the start
                                         table.                       */

    /* Point and Shoot Usage */
    UCHAR   UsingPointNShoot;      /* TRUE | FALSE                    */
    UCHAR   UsingNonPKColList;     /* TRUE | FALSE. The  Point and Shoot
                                      List is bas-ed on a Non-Primary
                                      Key set of columns.  Here PK
                                      columns as known to the corr.
                                      DBMS catalog.                   */
    UCHAR   UnionWithSelCrit;      /* TRUE | FALSE.  Union the rows
                                      selected via Point and Shoot with
                                      those selected via Start Table
                                      Where clause criteria.          */
    RDB_COL_LIST *  pPNSCols;      /* If UsingNonPKColList, this is the
                                      linked list of cols that make up
                                      a point and shoot value.  The
                                      columns are assumed to be in the
                                      proper order on the list.  The
                                      PKColSeq field is ignored.      */

    UCHAR   BuildXFPKIndex;        /* T | F.  Build a PK Index on the
                                      Extract File.  SyncPoint only.  */

    /* KSB - 03/17/2011 - RTC-22144
       Uplift the Maximum Row limit supported in Optim.
       Phase 1 - Limited to Archive, Insert, Delete, Convert and Load.*/
    unsigned long long    MaxRowsToPut;
                                   /* Max number of rows to put to the
                                      extract file.  0 implies there
                                      is no limit.                    */

    /* The following fields are used to support N of M Processing */
    wchar_t NofMKeyColumn[RDB_COL_NAME_LEN + 1]; /* N of M Column     */
    long    NumRowsPerValue;                 /* N Value 0 if not spec.*/
    long    NumValues;                       /* M Value 0 if not spec.*/

    /* The following fields help decide Table Scan versus Table Key
       Lookup access strategies.  The user tells us how often they
       want to have extract consider a Scan versus a Key Lookup to
       access a table.  The user can say never scan, rarely scan,
       or sometimes scan.  One of the following factors is choosen
       bas-ed on the user decision and the current situation; we have
       PKs to lookup and there is an appropriate index or FKs to lookup
       and an index is avail. If the following comparison is TRUE, the
       we use the Scan technique to fetch the rows:

            (NumKeysToLookup / NumRowsInTable) > Factor

      If a PK or FK is not backed by an index, we will do a scan unless
      the number of lookup keys divided by the number of rows is less
      than the corresponding ...WoIdx... factor.

      The Never factor should always be .99.  The comments suggest
      reasonable values for the other factors.                       */

    UCHAR   UserScanFactor;           /* DX_SCAN_NEVER,
                                         DX_SCAN_RARELY,    Default
                                         DX_SCAN_SOMETIMES            */
    float   ParWIdxNeverFactor;       /* Access Par with avail indexes*/
    float   ParWIdxRarelyFactor;      /*   "     .7                   */
    float   ParWIdxSometimesFactor;   /*   "     .5                   */
    float   ParWoIdxNeverFactor;      /* Access Par wo avail indexes  */
    float   ParWoIdxRarelyFactor;     /*   "     .01                  */
    float   ParWoIdxSometimesFactor;  /*   "     .002                 */
    float   ChlWIdxNeverFactor;       /* Access Chl with avail indexes*/
    float   ChlWIdxRarelyFactor;      /*   "     .4                   */
    float   ChlWIdxSometimesFactor;   /*   "     .3                   */
    float   ChlWoIdxNeverFactor;      /* Access Chl wo avail indexes  */
    float   ChlWoIdxRarelyFactor;     /*   "      .005                */
    float   ChlWoIdxSometimesFactor;  /*   "      .001                */

    /* The following two flags do not override the Force...TblAccess
       flags in the Input Rel structs.  They are only applied if a
       Rel flag is DX_FORCE_NONE.                                     */
    UCHAR   ForceKeyLookup;       /* T|F, Force Key lookup strategy   */
    UCHAR   ForceScan;            /* T|F, Force table scan strategy   */

    /* Some internal flags to force various execution actions */
    UCHAR   RelNumCharConvMode;  /* Special flag for controlling how
                                    Relationship processing should
                                    handle Numeric to Character
                                    conversions.
                                      DX_TRANS_NC_REL_STD
                                          Right justify num, zero fill
                                      DX_TRANS_NC_REL_ZAP1
                                          Right justify num, space fill
                                      DX_TRANS_NC_REL_ZAP2
                                          Left justify num, space fill
                                    This flag is ignored on
                                      DX_FUNC_EXEC calls.             */

    /* Execution Parameters *******************************************/
    /* The following parms are only used for DX_FUNC_COMPILE_AND_EXEC,
       DX_FUNC_EXEC calls                                             */

    /* The following is a code to indicate whether DDL is to be
       extracted or not.  Valid values are:
            DX_DATA_ONLY
            DX_DDL_ONLY
            DX_DATA_AND_DDL                                           */
    UCHAR   ExtractDDL;

    /* The following flag indicates whether XFREL structs are to be
       built and saved on the Extract File.                           */
    UCHAR   BuildXFRELs;      /* TRUE | FALSE                         */

    /* If DDL is to be extracted, then the following field points to
       a struct that defines what object types to extract and return
       counters for the numbers of the various object that were
       extracted.                                                     */
    DX_DDL_OBJECT_TYPE *   pDDLObjectTypes;

    /* If DDL is to be extracted, the following may point to a
       linked list of explicit DDL objects to extract.                */
    DX_EXPLICIT_OBJECT_LIST *   pExplicitDDLObjList;

    UCHAR *  pObjBlk;              /* Pointer to the Compiled Object
                                      Block. Only used for
                                      DX_FUNC_EXEC calls.             */
    UCHAR   MaxRowsOverrideFlag;   /* Override flag for MaxRowsToPut
                                      DX_OVERRIDE_NONE - No override
                                                         value
                                      DX_OVERRIDE_PRESENT - Following
                                           contains an override value */
    long    OverrideMaxRowsToPut;  /* Override max number of rows to
                                      put to the extract file.        */

    DX_PNS_LIST *    pPNSList;     /* Ptr to Point and Shoot List. NULL
                                      if Point and Shoot not defined. */

    /* The following fields are used to override N of M options set at
        compile time.                                                 */
    UCHAR   NofMOverrideFlag;      /* Override flag for MaxRowsToPut
                                      DX_OVERRIDE_NONE - No override
                                                         values
                                      DX_OVERRIDE_PRESENT - Following
                                           contains a set of override
                                           values                     */
    /* If an override is present, all values must be specified */
    wchar_t OverrideNofMKeyColumn[RDB_COL_NAME_LEN + 1];
    long    OverrideNumRowsPerValue;
    long    OverrideNumValues;


    /* Some DDL Extraction fields for future use */
    void *            pDDLExcpList;  /* Returned pointer of DDL objects
                                        not extracted because they
                                        referenced tables not in the
                                        input table list              */
    MEM_HEAP_HANDLE   hDDLExcpHeap;  /* Heap to use to alloc memory for
                                        DDL Object Exception List     */


    /* XF Builder fields */
    UCHAR    InputXFDDName[9];    /* Optional Input XF DDName.   */
                                  /* For SyncPoint.  If present, */
                                  /* for tables on this XF File, */
                                  /* rows are copied from the XF */
                                  /* and NOT fetched from the DB */
    UCHAR    ExtractDDName[9];    /* 8 char Overriding DDName    */
                                  /*  (w/terminating null)  for  */
                                  /*  Extract File. First char   */
                                  /*  must be '\0' for no        */
                                  /*  override!                  */
    XFPROD   XFProd;              /* product building XF         */
    UCHAR    ProductFlags;        /* Enabled products            */
    long     CSFlags;             /* client/server flags         */
    short    SplitIndexPercent;   /* When the percent of a row   */
                                  /*  taken by the index exceeds */
                                  /*  this value, a split index  */
                                  /*  rather than a redundant    */
                                  /*  index will be built        */
                                  /*  (SyncPoint)                */

    /* Function pointers to notify the caller of the extract process  */
    /* The following field is passed through to all of the following
       exit calls                                                     */
    void *     pProdAnchor;

    /* exit for progress popups for data extraction.  Only return codes
       expected from exit are PST_SUCCESS and PST_FAILED.             */
    /* KSB - 03/17/2011 - RTC-22144
       Uplift the Maximum Row limit supported in Optim */
    short     (*pNotifyExit) (PST_THANDLE,
                              void *,   /* pProdAnchor                */
                              wchar_t *,/* Table Name, Zero Dlm       */
                              short,    /* Processing constant, e.g.
                                           DX_PROCESSING_IN_PROGRESS  */
                              long,     /* # Rows extracted for table */
                              unsigned long long);    /* Tot # rows extracted       */

    /* exit for progress popups for DDL extraction */
    void     (*pDDLNotifyExit) (void *,            /* Product Ancchor */
                                CSSDDL_ProgressReport *);

    /* Error log exit - Called when XFB detects a duplicate row       */
    void      (*pErrorLogExit) (DX_LOG_CB *);

    /* The following field is polled during the Execution
       Phase to check if the calling product wants to abort the
       Extract.  A non-zero value in the byte pointed to by this
       field will abort the Extract.  If this field is NULL, no
       polling takes place.                                       */
    UCHAR *    pAbortSemaphore;


    /* Return Structures **********************************************/
    long        ObjBlkLen;        /* Returned length of the following
                                     Object Block.                    */
    UCHAR *     pCompileObjBlk;   /* Returned pointer to the Compile
                                     Object Block.  These block is one
                                     contiguous, relocatable chunk of
                                     character and binary data. It is
                                     the caller's responsibility to
                                     call RDB_MemFree() to release
                                     this block.  This parm is only
                                     used on DX_FUNC_COMPILE calls.   */
    RDB_MEM_HANDLE     hRDBMem;   /* Returned handle to be used to
                                     free the Compile Object Block
                                     This parm is only used on
                                     DX_FUNC_COMPILE calls.           */

    MEM_HEAP_HANDLE   hRtnHeap;      /* Heap to use to alloc memory for
                                        DX_OUTPUT_TABLE & DXPL structs.
                                        If this field is NULL, those
                                        structures are not built.     */
    DX_OUTPUT_TABLE * p1stRtnTbl;    /* A returned linked list of
                                        structs that specify the number
                                        of rows extracted for each
                                        table.  Returned after
                                        DX_FUNC_COMPILE_AND_EXEC and
                                        DX_FUNC_EXEC calls.           */
    DXPL_MAIN *       pRtnExplain;   /* A returned linked set of
                                        structs that specify the SHOW
                                        STEPS data.  Returned after
                                        DX_FUNC_SHOW_STEPS calls.     */
    UCHAR        BuildStats;         /* TRUE|FALSE, build a linked list
                                        of extract statistics structs */
    DX_STATS_STEP *  p1stStatsStep;  /* A returned linked list of
                                        statistics structures         */


    /* The following fields indicate the number of tables and rels
       logically untraversed bas-ed on the input table and rel
       definitions.  They are set after DX_FUNC_SHOW_STEPS,
       DX_FUNC_VALIDATE, DX_FUNC_COMPILE, and DX_FUNC_COMPILE_AND_EXEC
       calls                                                          */
    short   NumUnTravTbls;      /* Num of tables not traversed */
    short   NumUnTravRels;      /* Num of relationships not traversed */

    /* The following field indicates the number of tables in the
       DX_INPUT_TABLE list that have TblStatsAvail set to
       DX_STATS_NOT_RUN.  It is set during DX_FUNC_VALIDATE,
       DX_FUNC_COMPILE, and DX_FUNC_COMPILE_AND_EXEC calls.           */
    short   NumTblsWithNORunTblStats;

    /* KSB - 03/17/2011 - RTC-22144
       Uplift the Maximum Row limit supported in Optim */
    unsigned long long    TotRowsExtracted;
                                    /* Total number of rows on the
                                       Extract File                   */

    /* If an Extract of an XF, and Parms.ExtractDDL == DX_DATA_AND_DDL,
       this field contains a status of whether DDL was copied or not. */
    UCHAR   RtnCopyDDL;       /*  DX_DDL_COPIED,
                                  DX_NO_DDL_TO_COPY, or
                                  DX_MVS_DDL_NOT_COPIED */


    /* Optional Product Strings to be written to the Extract File     */
    wchar_t *         pProdString1;      /* An optional Product string
                                          to be written to teh extract
                                          file.                       */
    long            ProdString1Len;    /* Length of the above string  */
    wchar_t *       pProdString2;      /* Another optional Product str*/
    long            ProdString2Len;    /* Length of the above string  */
    wchar_t *       pProdString3;      /* Another optional Product str*/
    long            ProdString3Len;    /* Length of the above string  */
    wchar_t *       pProdString4;      /* Another optional Product str*/
    long            ProdString4Len;    /* Length of the above string  */
    wchar_t *       pProdString5;      /* Another optional Product str*/
    long            ProdString5Len;    /* Length of the above string  */
    wchar_t *       pProdString6;      /* Another optional Product str*/
    long            ProdString6Len;    /* Length of the above string  */
    wchar_t *       pProdString7;      /* Another optional Product str*/
    long            ProdString7Len;    /* Length of the above string  */
    wchar_t *       pProdString8;      /* Another optional Product str*/
    long            ProdString8Len;    /* Length of the above string  */

    /* The following fields deal with Archive Extracts */
    UCHAR  ArchiveExtract;    /* TRUE | FALSE, this extract is an
                                 Archive Extract.                        */
    long   ArchiveID;         /* Sequentially assigned Archive File ID.
                                 This field is filled in by Extract and
                                 is returned to the caller.               */
    UCHAR  DateTimeOfArchive[DX_DATETIME_LEN+1];
                            /* Date and Time the archive file was created
                               This field is filled in by Extract and
                               is returned to the caller.
                               In YYYY-MM-DD-HH.MM.SS format              */
    WCHAR  ArchiveFileName[RDB_DSN_NAME_LEN + 1];
                              /* Archive File Name                        */
    WCHAR  IndexFileName[RDB_DSN_NAME_LEN + 1];
                              /* Archive File's Index File                */
    UCHAR  ArchiveMediaFormat;   /* Media Type for the Archive File       */
    UCHAR  IndexMediaFormat;     /* Media Type for the Index File         */
    wchar_t GroupName[DX_GROUP_NAME_LEN+1];
                            /* Group the Archive File is a member of      */
    wchar_t UserID[RDB_USERID_LEN+1];
                            /* Workstation ID of the user who executed the
                               Archive                                    */
    wchar_t ServerName[DX_SERVER_NAME_LEN+1];
                          /* Server name used to produce the Archive File */
    wchar_t Description[DX_LONG_DESC_LEN+1];
                           /* User descriprion of Archive File            */
    wchar_t MachineName[DX_SERVER_NAME_LEN+1];
                           /* Machine on which Archive was prepared       */

    wchar_t *pBefExtrAct;  /* Ptr to Before Extract Process Action
                              Statement.  NULL if none.                   */
    wchar_t *pBefExtrActDBAliasName;  /* DBAlias for Before Extract
                                         Process Action.                  */
    UCHAR    BefExtrActErrControl;    /* DX_ACTION_ERR_STOP, ...          */
    UCHAR    BefExtrActHVChar;        /* Host Var Char for this action   */

    wchar_t *pAftExtrAct;  /* Ptr to After Extract Process Action
                              Statement.  NULL if none.                   */
    wchar_t *pAftExtrActDBAliasName;  /* DBAlias for After Extract
                                         Process Action.                  */
    UCHAR    AftExtrActErrControl;    /* DX_ACTION_ERR_STOP, ...          */
    UCHAR    AftExtrActHVChar;        /* Host Var Char for this action   */

    /* Parms to control mountable media and XF segmentation */
    UCHAR               fUseOSIOEx;    /* indicates that ONPB pointer    */
                                       /*  is valid.                     */
    OSIO_OpenExNewPB *  pONPB;         /* pointer to a struct to define  */
                                       /* the output media & segmentation*/

    unsigned long     ProductBuildVer;    /* Product Build Version       */
    unsigned short    ProductBuildSubVer; /* Product Build Sub Version   */
                                          /* Product Vuild String        */
    WCHAR             ProductBuildString[8 + 1];

    /* If fUseOSIOEx, then these are pointers to the resultant manifests */
    OSIO_Manifest *   pRtnManifestPrimary;
    OSIO_Manifest *   pRtnManifestSecondary;

    /*------------- segmented file parameters for input/restore XF -----*/
    UCHAR             fUseOSIOExForInputXF;  /* indicates that OEPB     */
                                             /* pointer is valid        */
    UCHAR             ReservedS[3];      /* padding                     */

    OSIO_OpenExExistingPB *
                      pOEPB;             /* pointer to OEPB             */
    /*------------- segmented file parameters --------------------------*/

    UCHAR             fGatherColStats; /* TRUE|FALSE Indicates whether
                                          column value cardinality and
                                          other statistics are to be
                                          gathered.                   */

    void *            hIXM;            /* If XtractFromXF, an optional
                                          Index file that corresponds
                                          to the XF.                  */

    UCHAR    PNSIndex;                /* One of the following codes
                                         indicating the Point and Shoot
                                         cols have a corresponding
                                         index.
                                         DX_PATH_INDEX_FULL - there
                                             is a full index.
                                         DX_PATH_INDEX_PARTIAL there
                                             is a partial index.
                                         DX_PATH_INDEX_NONE - there
                                             is no index..            */
    long     NotifyRowThreshold;     /* Number of rows to be extracted
                                        before calling the NotifyExit.
                                        0 indicates to use the default
                                        which is 100 rows             */
    long     NotifyTimeThreshold;    /* Elapsed time in seconds between
                                        calls to the Notify Exit.
                                        0 indicates to use the default
                                        which is 5 seconds.           */
    wchar_t *pFADSName;              /* File Access Data Structure name */
    long     MaxConn;                /* Maximum no of DBMS connections
                                        used during parallel processing */

    SEC_Token SecToken;              /* XFM security token              */

    DX_ARC_COLLECTION * pFirstArcCollection; /* List of Archive Collection
                                                names to which the archive
                                                will be added               */
    DSS_AFC_ERROR     * pDSSAddAFErr; /* Errors from Adding Archive file to collection */
    UCHAR             * DateOfDelete;      /* Flds to put into DSS_ARCHIVE_FILE_REC */
    WCHAR             * DeleteUserId;      //*
    short               DeletePasswordLen; //*
    UCHAR             * DeletePassword;    //*
    WCHAR             * DeleteDomainName;  //*


    DX_OUTPUT_REL    * p1stRtnRel;    /* A returned linked list of
                                        structs that provide information
                                        about the relationships used     */

    UCHAR           MultiByteRTDisp; // Round-trip error handling flag
                                     // Values are #defined below
    long            ExtractMaxAllocation; // This will be maximum total number of bytes to allocate for all buffers when running an extract.  Allowed values will be 64k to 1024k  (where k=1024)
    UCHAR           FileComp;             // Extract/Archive File Compressed

    long            SortThreshold;   // Value in Kb retrieved from Optim option to inform IXM when
                                     // to use MSort instead of PSort when sorting the index.
    UCHAR           AddPseudcolRRN;   /* TRUE | FALSE.  Should a pseudocolumn be added to each
                                      table in subset XF to hold the RRN (in the input AF) of
                                      the row to be deleted.                        */
    UCHAR           NoRICycleRevisit; /* TRUE | FALSE.  If TRUE, do not revisit tables for
                                      RI cycle.                                     */
	// KSB - 06/14/2010 - Begin - Added for inline Active Compression
	short           sActCompDefThreshold;          /* Active Compression Default Threshold */
    DXX_ACTIVE_COMPRESS* pFirstTableActCompBlk;    /* Active Compression blocks */
    BOOL            bActiveCompression;            /* Active compression enabled/disabled */
	// KSB - 06/14/2010 - End - Added for inline Active Compression
	double			DataByteCount; /* Uncompressed Data Byte Count in MB and Upto 3 decimal places */

    UCHAR           Reserve[50];
} DX_PARMS;

// Values for DX_PARMS->MultiByteRTDisp
#ifndef RDB_RTDISP_ABORT
 #define RDB_RTDISP_ABORT      'A'
 #define RDB_RTDISP_LOG        'L'
 #define RDB_RTDISP_IGNORE     'I'
#endif





/* Error Information Block */
typedef struct s_DXCA
{
    /* This field is set by the Caller, but Extract doesn't really
       use it.                                                       */
    DX_PARMS *  pParms;

    /* The following fields are set if the Return Code = DX_ERROR */
    short   ErrMsgNum;                 /* The type of error detected */
    wchar_t ErrTok1[DX_ERRTOK_LEN+1];  /* Err msg substitution word  */
    wchar_t ErrTok2[DX_ERRTOK_LEN+1];  /* Err msg substitution word  */
    wchar_t ErrTok3[DX_ERRTOK_LEN+1];  /* Err msg substitution word  */
    wchar_t ErrTok4[DX_ERRTOK_LEN+1];  /* Err msg substitution word  */
} DXCA;






/*--------------------------------------------------------------------*
 *  DXX Function Vector                                               *
 *--------------------------------------------------------------------*/
 typedef struct DXX_PubFunc
  {
    FRM_FootPrint         Header;      /* Standard footprint header   */
    short   (*pDXProcessInit)     (void);
    void    (*pDXProcessTerm)     (void);
    short   (*pDXThreadAct)       (void);
    void    (*pDXThreadTerm)      (void);

    short   (*pDXExtract) (UCHAR,         /* Func Code                */
                           DX_PARMS *,    /* Parm Block               */
                           DXCA *);       /* Communication Area       */

    short   (*pDXGetActColDefs) (UCHAR,   /* DBMS Type                */
                      MEM_HEAP_HANDLE,    /* MemHeap to alloc from    */
                      RDB_COL_LIST **,    /* Rtned Process ColList    */
                      RDB_COL_LIST **);   /* Rtned Table ColList      */

    /* Room for a total of 12 external functions                      */
    /* 6 external funcs defined so far                                */
    short   (*DummyFuncs1[6]) (void);

    /* Internal funcs */
    /* Room for a total of 4 internal functions                       */
    /* 0 internal funcs defined so far                                */
    short   (*DummyFuncs2[4]) (void);

 } DXX_PubFunc;

/*--------------------------------------------------------------------*
 *  Assist macros used to reference Relational DB Public Functions    *
 *--------------------------------------------------------------------*/
#define DXX_PROCESS_INIT     (*dxxgfloc->pDXProcessInit) ()
#define DXX_PROCESS_TERM     (*dxxgfloc->pDXProcessTerm) ()
#define DXX_THREAD_INIT      (*dxxgfloc->pDXThreadAct) ()
#define DXX_THREAD_TERM      (*dxxgfloc->pDXThreadTerm) ()

#define DX_Extract(a,b,c)    (*dxxgfloc->pDXExtract) (a,b,c)
#define DX_GetActionColDefs(a,b,c,d)                             \
                         (*dxxgfloc->pDXGetActColDefs)(a,b,c,d)


#ifdef PST_CPLUS
}
#endif
#endif
