define(["dojo/_base/declare",
        "dojo/_base/lang",
        "dijit/_Widget",        
        "dijit/_TemplatedMixin",
        "dijit/_WidgetsInTemplateMixin",
        "dojo/text!./templates/Wizard.html",
        "dojo/dom-style",
        "dojo/i18n!../nls/user_resources",
        "dijit/form/Button",
        "dijit/form/TextBox",
        "dijit/layout/StackContainer",
        "dijit/layout/BorderContainer",
        "dijit/layout/ContentPane",
        "idx/form/RadioButtonSet"],
		function(dDeclare,
				dLang,
				dWidget,
				dTemplatedMixin,
				dWidgetsInTemplateMixin,
				templateText,
				domStyle,
				resources) {
return dDeclare("admin.users.dialogs.Wizard", [dWidget,dTemplatedMixin,dWidgetsInTemplateMixin], 
{
	/**
	 * @private
	 */
	baseClass: "consoleWizard",

	/**
	 * The path to the widget template for the dijit._TemplatedMixin base class.
	 * 
	 * @private
	 * @constant
	 * @type String
	 */
	templateString: templateText,
	 
	/** 
	 * Allow users to add other components (e.g. hidden fields) under password field
	 * 
	 * @private
	 */
	isContainer: true,
	
	/**
     * Indicate if there are widgets in the template
     */
    widgetsInTemplate: true,
	
    /**
     * The back button label
     */
	backButtonLabel: resources["backButtonLabel"],
	/**
	 * The next button label
	 */
	nextButtonLabel: resources["nextButtonLabel"],
	/**
	 * The finish button label
	 */
	finishButtonLabel: resources["finishButtonLabel"],
	/**
	 * The cancel button label
	 */
	cancelButtonLabel: resources["cancelButtonLabel"],

	/**
	 * The wizard controller
	 */
	controller: null,
	
	/**
	 * Default page description if it's missing in the wizard page
	 */
	pageDescription:"Unknown description",
	
	/**
	 * Set the wizard controller
	 */
	_setControllerAttr:function (controller) {
		console.log("wizard.setController()");
		this.controller = controller;
		this.controller.setWizard(this);
	},
	
	attributeMap: dLang.delegate(dWidget.prototype.attributeMap, {
		pageDescription: {node: "pageDescriptionNode", type: "innerHTML"}
	}),
	
	/**
	 * The startup function
	 */
	startup: function(){
		this.inherited(arguments);
		this.connect(this.nextButton, "onClick", "_forward");
		this.connect(this.previousButton, "onClick", "_back");
		this.connect(this.cancelButton, "onClick", "_cancel");
		this.connect(this.finishButton, "onClick", "_finish");
		this._setPageTitleAndDescription();
		this._checkButtons();
		this.mainContainer.resize();
	},
	  
	/**
	 * Reset the content of this wizard - for reusable dialog
	 */
	resetWizard:function() {
		console.log("resetting wizard pages");
		var pages = this.pageContainerNode.getChildren();
		for (var i = 0; i < pages.length; i++) {
			if (typeof pages[i].init != "undefined") {
				pages[i].init();
			}
			else {
				console.log("unknown page");
				console.log(pages[i]);
			}
		}
		this.pageContainerNode.selectChild(pages[0]);
		this._checkButtons();
		this._setPageTitleAndDescription();
	  },
	  
    /**
     *Get the current page
     * @return the current page
     */
    getCurrentPage:function() {
        return this.pageContainerNode.selectedChildWidget;
    },

	/**
	 * Display the title and description of the current page
	 */
	_setPageTitleAndDescription:function() {
		var page = this.getCurrentPage();
	    //var title = page.getTitle();
	    var description = page.getDescription();
		if (description) {
			description = "<b>" + description + "</b>";
			this.set('pageDescription', description);
			domStyle.set(this.pageDescriptionNode, "visibility", "visible");
            domStyle.set(this.pageDescriptionNode, "display", "block");
		}
		else {
			domStyle.set(this.pageDescriptionNode, "visibility", "hidden");
            domStyle.set(this.pageDescriptionNode, "display", "none");
		}
		this.mainContainer.resize();
	},
	
	/**
	 * Update button states for the current page
	 */
	_checkButtons: function() {
		console.log("wizard._checkButtons()");
		var selectedPage = this.pageContainerNode.selectedChildWidget;
		var enableNext = selectedPage.isComplete();
		var enablePrevious = false;
		var canFinish = false;
		if (this.controller != null) {
			var nextPage = this.controller.getNextPage(selectedPage);
			var prevPage = this.controller.getPreviousPage(selectedPage);
			if (nextPage == null) {
				enableNext = false;
			}
			if (prevPage != null) {
				enablePrevious = true;
			}
			canFinish = this.controller.canFinish();
		}
		else {
			console.log("@@@controller is null!")
		}
		this.nextButton.set("disabled", !enableNext);
		this.previousButton.set("disabled", !enablePrevious);
		this.finishButton.set("disabled", !canFinish);
	},
	
	/**
	 * Cancel button handler
	 */
	_cancel: function() {
		this.onCancel();
	},
	
	/**
	 * Finish button handler
	 */
	_finish: function() {
		console.log("finished pressed");
		this.onFinish();
	},
	
	/**
	 * Next button handler
	 */
	_forward: function(){
		var selectedPage = this.pageContainerNode.selectedChildWidget;		
		if (typeof selectedPage.completePage != "undefined") {
			if (!selectedPage.completePage()) {
				return;
			}
		}
		if (this.controller != null) {
			var nextPage = this.controller.getNextPage(selectedPage);
			console.log("nextPage:", nextPage);
			if (nextPage != null) {
				this.pageContainerNode.selectChild(nextPage);
				if (typeof nextPage.pageDisplayed != "undefined") {
					nextPage.pageDisplayed();
				}
			}
		}
		this._setPageTitleAndDescription();
		this._checkButtons();
	},
	
	/**
	 * Back button handler
	 */
	_back: function() {
		var selectedPage = this.pageContainerNode.selectedChildWidget;
		if (this.controller != null) {
			var prevPage = this.controller.getPreviousPage(selectedPage);
			if (prevPage != null) {
				this.pageContainerNode.selectChild(prevPage);
				if (typeof prevPage.pageDisplayed != "undefined") {
					prevPage.pageDisplayed();
				}
			}
		}
		this._setPageTitleAndDescription();
		this._checkButtons();
	},
	
	/**
	 * Place holder for onCancel
	 */
	onCancel:function() {
		  
	},
	
	/**
	 * Place holder for onFinish
	 */
	onFinish: function() {
		//user of this wizard should override this function
	}
});

});